#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
GUEST plugin commands
"""

from poppy.core.command import Command

from roc.guest.tools import valid_time, valid_single_file
from roc.guest.constants import SCOS_HEADER_BYTES, IDB_SOURCE
from roc.guest.tasks import \
    xml_to_test, txt_to_test, \
    test_to_xml, test_to_txt, \
    mebdb_to_test, test_to_mebdb, \
    raw_to_tmraw, raw_to_tcreport
from roc.guest.tasks.parse_test_packets import ParseTestPackets
from roc.guest.tasks import L0ToTest, TestToL0
from roc.guest.tasks.rocdb import clear_test

__all__ = []


class GuestCommands(Command):
    """
    Manage the commands relative to the roc.guest module.
    """
    __command__ = 'guest'
    __command_name__ = 'guest'
    __parent__ = 'master'
    __parent_arguments__ = ['base']
    __help__ = """
        Commands relative to the roc.guest module.
    """

    def add_arguments(self, parser):
        # Option to filter test log data by testname pattern
        parser.add_argument(
            '-p', '--pattern',
            nargs=1,
            help="""
            Testname filter pattern.
            """,
            type=str,
        )

        # Option to set a lower time interval limit
        parser.add_argument(
            '--start-time',
            help="""
            Minimum of the date/time interval to export [YYYY-mm-ddTHH:MM:SS].
            """,
            default=None,
            type=valid_time,
        )

        # Option to set a upper time interval limit
        parser.add_argument(
            '--end-time',
            help="""
            Maximum of the date/time interval to export [YYYY-mm-ddTHH:MM:SS].
            """,
            default=None,
            type=valid_time,
        )

        # Optional argument to only process "Terminated" Test
        parser.add_argument(
            '-T', '--terminated',
            help="""
            Only process terminated tests
            """,
            action='store_true',
        )

        # specify the IDB version to use
        parser.add_argument(
            '--idb-version',
            help='IDB version to use.',
            nargs=1,
            default=None,
        )

        # specify the IDB source to use
        parser.add_argument(
            '--idb-source',
            help='IDB source to use (MIB, SRDB or PALISADE).',
            nargs=1,
            default=[IDB_SOURCE],
        )

        # Provide output test name
        parser.add_argument(
            '--out-test-name',
            type=str,
            help='Output test name.',
        )

        # Provide output test description
        parser.add_argument(
            '--out-test-descr',
            type=str,
            help='Output test description.',
        )

        # Remove SCOS2000 header in the binary packet
        parser.add_argument(
            '--scos-header', nargs=1,
            type=int, default=[None],
            help='Remove the '
                 'SCOS2000 header in the packet(s).'
                 ' (Value for MOC DDS should be {0} bytes.)'.format(SCOS_HEADER_BYTES)
        )

        # no-spice keyword to force not use of SPICE kernels
        parser.add_argument(
            '--no-spice', action='store_true',
            default=False,
            help='If False, then use SPICE kernels to compute time (SPICE_KERNEL_PATH env. variable must be defined)'
        )


class MebDbToXmlCommand(Command):
    """
     Command to extract test log data from a MEB GSE database and convert it into XML files (one file per test).
     """
    __command__ = 'guest_mebdb_to_xml'
    __command_name__ = 'mebdb_to_xml'
    __parent__ = 'guest'
    __parent_arguments__ = ['base']
    __help__ = """
         Command to export test log data from a MEB GSE database (one XML file per test).
     """

    def setup_tasks(self, pipeline):
        # starting point of the pipeline
        start = mebdb_to_test()

        # task starting the loop
        end = test_to_xml()

        # create the tasks workflow
        pipeline | start | end

        # define the start points of the pipeline
        pipeline.start = start
        pipeline.end = end

        # create a loop
        pipeline.loop(start, end, start.generator)


class XmlToMebDbCommand(Command):
    __command__ = 'guest_xml_to_mebdb'
    __command_name__ = 'xml_to_mebdb'
    __parent__ = 'guest'
    __parent_arguments__ = ['base']
    __help__ = 'Command to import a given test log XML into a MEB GSE database.'

    def add_arguments(self, parser):
        parser.add_argument(
            'test_log_xml',
            nargs=1,
            type=str,
            default=None,
            help='Input test log XML file to import'
        )

    def setup_tasks(self, pipeline):
        # Parse input file, then import data inside MEB GSE database
        start = xml_to_test()
        end = test_to_mebdb()

        # set the pipeline workflow
        pipeline | start | end

        # define the start points of the pipeline
        pipeline.start = start


class TxtToXmlCommand(Command):
    """
    Manage command to convert an input ADS GSE text format file
    into a MEB GSE XML test log file.
    """
    __command__ = 'guest_txt_to_xml'
    __command_name__ = 'txt_to_xml'
    __parent__ = 'guest'
    __parent_arguments__ = ['base']
    __help__ = """
        Manage command to convert an input ADS GSE text format file
    into a MEB GSE XML test log file.
    """

    def add_arguments(self, parser):
        # add lstable argument
        #        LSTableMixin.add_arguments(parser)

        # path to input ADS file
        parser.add_argument(
            'ads_gse_txt',
            help="""
            The input ADS GSE text format file to convert.
            """,
            type=valid_single_file,
        )

        # path to input ADS file
        parser.add_argument(
            '--output-file',
            help="""
            The output test log XML file.
            """,
            type=str,
            default=None,
        )

        # Remove header in bytes from the binary packets
        parser.add_argument(
            '--remove-header',
            help='Remove a header (in bytes) from the binary packet(s)',
            type=int,
            default=None,
        )

    def setup_tasks(self, pipeline):
        # Import task
        from roc.rpl.tasks import IdentifyPackets as identify_packets

        # Define start/end tasks
        start = txt_to_test()

        # Create topology
        pipeline | start | identify_packets() | test_to_xml()

        # define the start points of the pipeline
        pipeline.start = start


class XmlToDdsCommand(Command):
    """
    Command to convert an input MEB GSE test log XML format file
    into MOC DDS TmRaw and/or TcReport XML file(s).
    """
    __command__ = 'guest_xml_to_dds'
    __command_name__ = 'xml_to_dds'
    __parent__ = 'guest'
    __parent_arguments__ = ['base']
    __help__ = """
        Command to convert an input MEB GSE test log XML format file
        into MOC DDS TmRaw and/or TcReport XML file(s).
    """

    def add_arguments(self, parser):
        # add lstable argument
        #        LSTableMixin.add_arguments(parser)

        # path to input ADS file
        parser.add_argument(
            'test_log_xml',
            help="""
            The input MEB GSE Test log XML format file to convert.
            """,
            type=valid_single_file,
        )

        # Remove header in bytes from the binary packets
        parser.add_argument(
            '--scos-header-size',
            help='Bytes length of the dummy SCOS header to add in the DDS binary packet(s)',
            type=int,
            default=None,
        )

        # Name of the output DDS tmraw xml file
        parser.add_argument(
            '--output-tmraw-xml',
            help='Name of the output DDS tmraw xml file',
            type=str,
        )

        # Name of the output DDS tcreport xml file
        parser.add_argument(
            '--output-tcreport-xml',
            help='Name of the output DDS tcreport xml file',
            type=str,
        )

    def setup_tasks(self, pipeline):
        # Import task
        from roc.rpl.tasks import IdentifyPackets as identify_packets

        # Define start task
        start = xml_to_test()

        # Create topology
        pipeline | start | identify_packets() | raw_to_tmraw() | raw_to_tcreport()

        # define the start point of the pipeline
        pipeline.start = start


class XmlToTxtCommand(Command):
    """
    Manage command to convert an input MEB GSE text log XML file into
    a ADS GSE text format file.
    """
    __command__ = 'guest_xml_to_txt'
    __command_name__ = 'xml_to_txt'
    __parent__ = 'guest'
    __parent_arguments__ = ['base']
    __help__ = """
        Command to convert an input MEB GSE text log XML file into
    a ADS GSE text format file.
    """

    def add_arguments(self, parser):
        # add lstable argument
        #        LSTableMixin.add_arguments(parser)

        # path to input ADS file
        parser.add_argument(
            'test_log_xml',
            help="""
            The input MEB GSE test log XML file to convert.
            """,
            type=valid_single_file,
        )

        # path to input ADS file
        parser.add_argument(
            '--output-file',
            help="""
            The output ADS text file.
            """,
            type=str, default=None,
        )

    def setup_tasks(self, pipeline):
        start = xml_to_test()
        end = test_to_txt()

        # Create topology
        pipeline | start | end

        # define the start points of the pipeline
        pipeline.start = start


class ClearDbCommand(Command):
    """
    Manage command to clear a given test
    inside the ROC database,
    providing the test name and uuid.
    """
    __command__ = 'guest_clear'
    __command_name__ = 'clear'
    __parent__ = 'guest'
    __parent_arguments__ = ['guest']
    __help__ = """
    Command to clear a test inside the ROC database.
    """

    def add_arguments(self, parser):
        # the name of the test
        parser.add_argument(
            '--test-name',
            nargs=1,
            help="""
            The name of the test to remove.
            """,
            type=str,
            default=[None],
        )

        # the UUID
        parser.add_argument(
            '--test-uuid',
            nargs=1,
            help="""
            The UUID of the test to remove.
            """,
            type=str,
            default=[None],
        )

        # Clear all tests
        parser.add_argument(
            '--clear-all',
            help="""
            Clear all tests in the ROC database (USE WITH CAUTION!).
            """,
            action='store_true',
        )

    def setup_tasks(self, pipeline):
        # starting task
        start = clear_test()

        # Set test terminated status off
        pipeline.properties.test_terminated = False

        # Set input arguments
        pipeline.properties.test_name = pipeline.properties.test_name[0]
        pipeline.properties.test_uuid = pipeline.properties.test_uuid[0]

        # create the tasks and their dependencies
        pipeline | start

        # define the start points of the pipeline
        pipeline.start = start


class ClearDbFromXmlCommand(Command):
    """
    Manage command to clear a given test
    inside the ROC database,
    providing the test log XML file.
    """
    __command__ = 'guest_clear_xml'
    __command_name__ = 'clear_from_xml'
    __parent__ = 'guest'
    __parent_arguments__ = ['base']
    __help__ = """
    Command to clear a test inside the ROC database from
    its test log XML file.
    """

    def add_arguments(self, parser):
        # the name of the test
        parser.add_argument(
            'test_log_xml',
            help="""
            The XML file the test to remove.
            """,
            type=str,
        )

    def setup_tasks(self, pipeline):
        # starting task
        start = xml_to_test()
        end = clear_test()

        # Set test terminated status off
        pipeline.properties.test_terminated = False

        # create the tasks and their dependencies
        pipeline | start | end

        # define the start points of the pipeline
        pipeline.start = start


class L0ToXmlCommand(Command):
    __command__ = 'guest_l0_to_xml'
    __command_name__ = 'l0_to_xml'
    __parent__ = 'guest'
    __parent_arguments__ = ['base']
    __help__ = 'Command to convert a given RPW L0 hd5 file into a MEB GSE test log XML file'

    def add_arguments(self, parser):
        parser.add_argument(
            'l0_file',
            type=str,
            nargs=1,
            help='Input RPW L0 hd5 file to convert'
        )

        parser.add_argument('--test-log-xml',
                            nargs=1,
                            default=None,
                            type=str,
                            help='Full path of the output test log XML file'
                            )

    def setup_tasks(self, pipeline):
        # Parse input file, then convert it into output test log XML
        start = L0ToTest()
        end = test_to_xml()

        # set the pipeline workflow
        pipeline | start | end

        # define the start points of the pipeline
        pipeline.start = start


class FromXmlCommand(Command):
    __command__ = 'guest_testlog_to_l0'
    __command_name__ = 'testlog_to_l0'
    __parent__ = 'guest'
    __parent_arguments__ = ['base']
    __help__ = 'Command to generate RPW L0 files from a given input MEB GSE test log XML file'

    def add_arguments(self, parser):
        parser.add_argument(
            '--test-log-xml',
            required=True,
            type=str,
            nargs=1,
            help='Input RPW MEB GSE test log XML file to process'
        )

    def setup_tasks(self, pipeline):
        # Parse input file, then convert it into output test log XML
        start = xml_to_test()
        end = TestToL0()

        # set the pipeline workflow
        pipeline | start | ParseTestPackets() | end

        # define the start points of the pipeline
        pipeline.start = start



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Unit tests for roc.guest plugin.
"""

import os
import tempfile
import subprocess

import unittest.mock as mock

from poppy.core.generic.requests import download_file
from poppy.core.logger import logger
from poppy.core.test import CommandTestCase
from poppy.core.db.connector import Connector

from roc.guest.models.meb_gse_data.event import Event
from roc.guest.exceptions import MebDbTransactionError


class TestGuest(CommandTestCase):
    base_url = \
        'https://rpw.lesia.obspm.fr/roc/data/private/' \
        'devtest/roc/test_data/rgts/guest'
    # test credentials
    username = 'roctest'
    password = None

    @classmethod
    def setup_class(cls):
        """
        Setup credentials
        """

        try:
            cls.password = os.environ['ROC_TEST_PASSWORD']
        except KeyError:
            raise KeyError('You have to define the test user password '
                           'using the "ROC_TEST_PASSWORD" environment'
                           'variable ')

    def setup_method(self, method):
        super().setup_method(method)

        self.tmp_dir_path = tempfile.mkdtemp()

    def load_manifest_file(self, manifest_filepath, manifest_file_url, auth=None):

        download_file(manifest_filepath, manifest_file_url, auth=auth)

        with open(manifest_filepath) as manifest_file:
            for line in manifest_file:
                yield line.strip('\n\r')

        os.remove(manifest_filepath)

    def get_files(self, name_test, category=None):
        categories = ['input', 'expected_output']
        if category not in categories:
            raise ValueError('Invalid category. Expected one of: %s' % categories)

        auth = (self.username, self.password)

        dir_path = os.path.join(self.tmp_dir_path, category)
        os.makedirs(dir_path, exist_ok=True)
        manifest_filepath = os.path.join(dir_path, 'manifest.txt')
        manifest_file_url = f'{self.base_url}/{name_test}/{category}s/manifest.txt'
        file_list = list(self.load_manifest_file(manifest_filepath,
                                                 manifest_file_url, auth=auth))

        for relative_filepath in file_list:
            # skip empty strings
            if not relative_filepath:
                continue

            # get the complete filepath
            filepath = os.path.join(dir_path, relative_filepath)
            os.makedirs(os.path.dirname(filepath), exist_ok=True)
            current_url = f'{self.base_url}/{name_test}/' \
                          f'{category}s/{relative_filepath}'
            download_file(filepath,
                          current_url,
                          auth=auth)

        return dir_path, file_list

    def get_inputs(self, name_test):
        return self.get_files(name_test, category='input')

    def get_expected_outputs(self, name_test):
        return self.get_files(name_test, category='expected_output')

    # @pytest.mark.parametrize("idb_source,idb_version", [
    #     ("MIB", "20190624"),
    #     ("PALISADE", "4.3.3_MEB_PFM"),
    # ])
    # def test_from_xml(self, idb_source, idb_version):
    #     from poppy.core.conf import Settings

    #     input_dir_path, inputs = self.get_inputs('from_xml')
    #     expected_output_dir_path, expected_outputs = self.get_expected_outputs('from_xml')

    #     generated_output_dir_path = os.path.join(self.tmp_dir_path, 'generated_output')
    #     os.makedirs(generated_output_dir_path, exist_ok=True)

    #     # initialize the main command
    #     main_command = ['pop', "guest",
    #                     "--idb-version", idb_version,
    #                     "--idb-source", idb_source,
    #                     "from_xml",
    #                     "--xml-test-log", os.path.join(input_dir_path, inputs[0]),
    #                     "--output-dir", generated_output_dir_path,
    #                     "-ll", "INFO"]

    #     # define the required plugins
    #     plugin_list = ['poppy.pop', 'roc.idb', 'roc.rpl', 'roc.guest', 'roc.film']

    #     # run the command
    #     # force the value of the plugin list
    #     with mock.patch.object(Settings, 'configure',
    #                            autospec=True,
    #                            side_effect=self.mock_configure_settings(dictionary={'PLUGINS': plugin_list})):
    #         self.run_command('pop db upgrade heads -ll INFO')
    #         self.run_command(['pop', '-ll', 'INFO', 'idb', 'install', '-s', idb_source, '-v', idb_version, '--load'])
    #         self.run_command(main_command)

    #     # compare directory content
    #     dirs_cmp = filecmp.dircmp(generated_output_dir_path,
    #                               expected_output_dir_path)

    #     dirs_cmp.report()

    #     # ensure that we have the same files in both directories
    #     assert (len(dirs_cmp.left_only) == 0) and (len(dirs_cmp.right_only) == 0)

    #     for filename in self.get_diff_files(dirs_cmp):
    #         # compare only cdf files with differences
    #         if filename.endswith('.cdf'):
    #             # use cdf compare to compute the differences between expected output and the command output
    #             result = cdf_compare(
    #                 os.path.join(generated_output_dir_path, filename),
    #                 os.path.join(expected_output_dir_path, filename),
    #                 list_ignore_gatt=[
    #                     'File_ID', 'File_UUID',
    #                     'Generation_date',
    #                     'Pipeline_version',
    #                     'Software_version',
    #                     'IDB_version'
    #                 ]
    #             )

    #             # compare the difference dict with the expected one
    #             if result:
    #                 logger.error(f'Differences between expected output and the command output: {pformat(result)}')

    #             assert result == {}

    # @pytest.mark.parametrize("idb_source,idb_version", [
    #     ("PALISADE", "4.3.3_MEB_PFM"),
    # ])
    # def test_xml_to_dds(self, idb_source, idb_version):
    #     from poppy.core.conf import Settings

    #     input_dir_path, inputs = self.get_inputs('from_xml')
    #     expected_output_dir_path, expected_outputs = self.get_expected_outputs('from_xml')

    #     generated_output_dir_path = os.path.join(self.tmp_dir_path, 'generated_output')
    #     os.makedirs(generated_output_dir_path, exist_ok=True)

    #     # initialize the main command
    #     main_command = ['pop', "guest",
    #                     "--idb-version", idb_version,
    #                     "--idb-source", idb_source,
    #                     "xml_to_dds",
    #                     "--scos-header-size", '76',
    #                     os.path.join(input_dir_path, inputs[0]),
    #                     "--output-dir", generated_output_dir_path,
    #                     "-ll", "INFO"]

    #     # define the required plugins
    #     plugin_list = ['poppy.pop', 'roc.idb', 'roc.rpl', 'roc.guest', 'roc.film']

    #     # run the command
    #     # force the value of the plugin list
    #     with mock.patch.object(Settings, 'configure',
    #                            autospec=True,
    #                            side_effect=self.mock_configure_settings(dictionary={'PLUGINS': plugin_list})):
    #         self.run_command('pop db upgrade heads -ll INFO')
    #         self.run_command(['pop', '-ll', 'INFO', 'idb', 'install', '-s', idb_source, '-v', idb_version, '--load'])
    #         self.run_command(main_command)

    #     # compare directory content
    #     dirs_cmp = filecmp.dircmp(generated_output_dir_path,
    #                               expected_output_dir_path)

    #     dirs_cmp.report()

    #     # ensure that we have the same files in both directories
    #     assert (len(dirs_cmp.left_only) == 0) and (len(dirs_cmp.right_only) == 0)

    #     # TODO - Add output xml comparison
    #     #for filename in self.get_diff_files(dirs_cmp):
    #         # compare only cdf files with differences

    # def get_diff_files(self, dirs_cmp, path=''):
    #     for name in dirs_cmp.diff_files:
    #         yield os.path.join(path, name)
    #     for parent, sub_dirs_cmp in dirs_cmp.subdirs.items():
    #         for filepath in self.get_diff_files(sub_dirs_cmp, path=os.path.join(path, parent)):
    #             yield filepath

    # def teardown_method(self, method):
    #     """
    #     Method called immediately after the test method has been called and the result recorded.

    #     This is called even if the test method raised an exception.

    #     :param method: the test method
    #     :return:
    #     """

    #     # rollback the database
    #     super().teardown_method(method)

    #     # clear the downloaded files
    # #     shutil.rmtree(self.tmp_dir_path)

    # def findDiff(self, xml1, xml2):
    #     for i in range(0, len(xml1)):
    #         if xml1[i] != xml2[i]:
    #             print(f'Difference found at pos {i}')
    #             if i > 20:
    #                 start = i-20
    #             else:
    #                 start = i
    #             end = i+50
    #             print(f'{xml1[start:end]} != {xml2[start:end]}')
    #             break

    def test_xml_to_mebdb(self):
        from poppy.core.conf import Settings

        input_dir_path, inputs = self.get_inputs('xml_to_mebdb')

        # initialize the main command
        main_command = 'pop guest xml_to_mebdb ' \
                       f'{os.path.join(input_dir_path, inputs[0])} '

        # define the required plugins
        # plugin_list = ['poppy.pop', 'roc.guest']

        # run the command
        subprocess.run(main_command, shell=True, check=True, timeout=36000)

        connector = Connector.manager[Settings.MEB_DATABASE]
        database = connector.get_database()

        # check the database is connected
        database.connectDatabase()

        # get the selected database from the MEB connector
        meb = connector.selected

        # connect to the database
        meb.connectDatabase()

        # check that the connection is made
        if not meb.connected:
            message = '{0} is not connected'.format(meb)
            logger.error(message)
            raise MebDbTransactionError(message)

        # Insert test log TestDescriptor data into the MEB GSE database
        with meb.query_context() as session:
            row = session.query(Event).count()

        assert (row == 59882)

    def test_l0_to_xml(self):
        from poppy.core.conf import Settings
        # import h5py

        input_dir_path, inputs = self.get_inputs('l0_to_xml')
        expected_output_dir_path, expected_outputs = self.get_expected_outputs('l0_to_xml')

        generated_output_dir_path = os.path.join(self.tmp_dir_path, 'generated_output')
        os.makedirs(generated_output_dir_path, exist_ok=True)

        # initialize the main command
        output_xml = os.path.join(generated_output_dir_path,
                                  os.path.splitext(inputs[0])[0] + '.xml')
        main_command = f'pop guest l0_to_xml {os.path.join(input_dir_path, inputs[0])} ' \
                       f'--test-log-xml {output_xml}'

        # define the required plugins
        plugin_list = ['poppy.pop', 'roc.idb', 'roc.guest']

        # run the command
        # force the value of the plugin list
        with mock.patch.object(Settings, 'configure',
                               autospec=True,
                               side_effect=self.mock_configure_settings(
                                   dictionary={'PLUGINS': plugin_list})):
            self.run_command(main_command)

        xmlOutputGenerated = XmlReader(generated_output_dir_path)
        xmlOutputExpected = XmlReader(expected_output_dir_path)

        # Remove some metadata defined "on-the-fly"
        xmlOutputGenerated.removeMarkup(['TestLogDescriptor', 'TestUUID'])
        xmlOutputExpected.removeMarkup(['TestLogDescriptor', 'TestUUID'])

        # Compare expected output XML and output XML
        assert (xmlOutputGenerated.file == xmlOutputExpected.file)


class XmlReader:
    file = ''

    def __init__(self, path_to_dir):
        for file in path_to_dir:
            if os.path.isfile(file):
                f = open(file, 'r')
                self.file = f.read()

    def removeMarkup(self, list_markup):
        for name_markup in list_markup:
            print(f'Before len : {len(self.file)}')
            start_index, end_index = self.findIndexMarkup(name_markup)
            str_to_remove = self.file[start_index:end_index]
            print(f'Removing : {str_to_remove}')
            self.file = self.file.replace(str_to_remove, '')
            print(f'After len : {len(self.file)}')

    def findIndexMarkup(self, name_markup):
        start_index = self.file.find(f'<{name_markup}>')
        end_index = self.file.find(f'</{name_markup}>')
        return start_index, end_index + len(f'</{name_markup}>')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""GUEST plugin constant variables."""
import os
from datetime import datetime

from jinja2 import Environment, FileSystemLoader

__all__ = [
    'ADS_GSE_STRFTIME',
    'INVALID_PACKET_TIME',
    'TESTLOG_STRFTIME',
    'CURDIR',
    'JINJA_TEMPLATE_DIR',
    'JENV',
    'TESTLOG_TEMPLATE',
    'TIME_RANGE_STRFORMAT',
    'TIME_ISO_STRFORMAT',
    'INPUT_TIME_STRFORMAT',
    'TESTLOG_EVENTDATE_STRFORMAT',
    'TESTLOG_EVENTTIME_STRFORMAT',
    'INPUT_DATETIME_STRFTIME',
    'INVALID_ADS_TIME',
    'TCREPORT_STRTFORMAT',
    'INVALID_UTC_DATETIME',
    'INVALID_UTC_TIME',
    'SCOS_HEADER_BYTES',
    'DATA_VERSION',
    'IDB_SOURCE',
]

# output format for string time in ADS GSE text file (e.g., 2018-08-06T14:15:47.375Z)
ADS_GSE_STRFTIME = '%Y-%m-%dT%H:%M:%S.%fZ'

# Time value for invalid packet
INVALID_PACKET_TIME = '0000-00-00T00:00:00.000Z'

# Time iso format
TIME_ISO_STRFORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'

# TC report time format
TCREPORT_STRTFORMAT = TIME_ISO_STRFORMAT

# value for invalid UTC Time
INVALID_UTC_TIME = '2000-01-01T00:00:00.000000Z'
INVALID_UTC_DATETIME = datetime.strptime(INVALID_UTC_TIME,
                                         TCREPORT_STRTFORMAT)

INPUT_DATETIME_STRFTIME = '%Y-%m-%dT%H:%M:%S'
TESTLOG_STRFTIME = '%Y-%m-%dT%H:%M:%S'

# Setup jinja2 environment
CURDIR = os.path.abspath(os.path.dirname(__file__))
JINJA_TEMPLATE_DIR = os.path.join(CURDIR, 'templates')
JENV = Environment(loader=FileSystemLoader(JINJA_TEMPLATE_DIR))
# Jinja2 template for GSE test log XML
TESTLOG_TEMPLATE = 'gse_test_log.tpl'

TIME_RANGE_STRFORMAT = '%Y%m%dT%H%M%S'
INPUT_TIME_STRFORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
TESTLOG_EVENTDATE_STRFORMAT = '%Y-%m-%d'
TESTLOG_EVENTTIME_STRFORMAT = '%H:%M:%S.%f'

# Invalid line in the ADS GSE text file
INVALID_ADS_TIME = '0000-00-00T00:00:00.000Z'

# Length in bytes of the TM SCOS HEADER
SCOS_HEADER_BYTES = 76

# DEfault data version
DATA_VERSION = '01'

# Default IDB source
IDB_SOURCE = 'PALISADE'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""GUEST plugin tasks related to MEB GSE test log XML handling."""

import os
import sys
import getpass
from shutil import copyfile as copy
from datetime import datetime

try:
    from poppy.core.logger import logger
    from poppy.core.db.connector import Connector
    from poppy.core.db.handlers import get_or_create_with_info
    from poppy.core.task import Task
    from poppy.core.target import FileTarget, PyObjectTarget
    from poppy.core.db.dry_runner import DryRunner
except:
    sys.exit('POPPy framework seems to not be installed properly!')

from roc.guest.guest import Test
from roc.guest.exceptions import ParseTestXmlError, GuestNoInputError
from roc.guest.constants import JENV, TESTLOG_TEMPLATE, TESTLOG_STRFTIME, \
    TESTLOG_EVENTDATE_STRFORMAT, TESTLOG_EVENTTIME_STRFORMAT

__all__ = ['xml_to_test', 'test_to_xml', 'copy_xml']

def get_test_log_xml(pipeline):
    # Pass the test_log_xml argument as the value of the input target test_log_xml of xml_to_test task
    try:
        return pipeline.args.test_log_xml[0]
    except:
        pass

@FileTarget.input(identifier='test_log_xml', filepath=get_test_log_xml)
@PyObjectTarget.output(identifier='raw_data')
@Task.as_task(plugin_name='roc.guest', name='xml_to_test')
def xml_to_test(task):
    """
    Parse test log XML and save content in a Test object.

    Parse header of the xml test log file
    """
    # Get test log file path

    test_file = task.inputs['test_log_xml'].filepath

    # Create test_log object
    test_log = Test.from_testlog_xml(test_file)

    # Check if test status is "Terminated"
    if ('terminated' in task.pipeline.args and
            task.pipeline.args.terminated == True and
            test_log.status != 'Terminated'):
        msg = '{0} is not terminated!'.format(test_file)
        logger.error(msg)
        raise ParseTestXmlError(msg)

    # TODO - improve this part to include packet_list setting into the Test class directly
    # raw data packets are stored in a list
    nevents = len(test_log.events)
    raw_binary_packets = [None] * nevents
    for i, event in enumerate(test_log.events):
        packet_type = event['EventType'][:2]

        # Only keep time for TC
        # (TM EventData-EventTime is not packet time creation in UTC)
        if packet_type == 'TM':
            utc_time = None
            ack_acc_state = None
            ack_exe_state = None
        else:
            utc_time = datetime.strptime(
                event['EventDate'] + 'T' + event['EventTime'],
                TESTLOG_EVENTDATE_STRFORMAT + 'T' + TESTLOG_EVENTTIME_STRFORMAT)
            if event.get('Status', 'UNKNOWN') == 'OK':
                ack_exe_state = 'PASSED'
                ack_acc_state = 'PASSED'
            elif event.get('Status', 'UNKNOWN') == 'NOK':
                ack_exe_state = 'FAILED'
                ack_acc_state = 'FAILED'
            else:
                ack_exe_state = 'UNKNOWN'
                ack_acc_state = 'UNKNOWN'

        raw_binary_packets[i] = {
            'binary': event['Data'],
            'palisade_id': event['Name'],
            # 'srdb_id': palisade_metadata[event["Name"]]['srdb_id'],
            'description': event['Name'],
            'category': event['Category'],
            'type': packet_type,
            'utc_time': utc_time,
            'ack_acc_state': ack_acc_state,
            'ack_exe_state': ack_exe_state,
        }

    # save the packet list in the test object
    test_log.packet_list = raw_binary_packets

    logger.info(f'{len(test_log.packet_list)} packets extracted from {test_file}')

    # Store test log in the pipeline
    task.outputs['raw_data'].value = test_log

@PyObjectTarget.input(identifier='raw_data')
@FileTarget.output(identifier='test_log_xml')
@Task.as_task(plugin_name='roc.guest', name='test_to_xml')
def test_to_xml(task):
    """
    Create an output XML file from the test log object stored in the pipeline properties
    :param task:
    :return:
    """

    # Retrieve input test log data
    try:
        # Get input test log object
        test_log = task.inputs['raw_data'].value
    except:
        logger.error('No input test_log data found for test_to_xml task!')
        return

    # Get data version (if any)
    data_version = task.pipeline.get('data_version', default=[None])[0]

    # Get overwrite argument or define it
    overwrite = task.pipeline.get('overwrite', default=False, args=True)

    if len(test_log.events) == 0 and len(test_log.packet_list) == 0:
        logger.warning('No event found in input raw_data')
    elif len(test_log.events) == 0 and len(test_log.packet_list) != 0:
        if test_log.packet_parser.idb_source == 'PALISADE':
            palisade_version = test_log.packet_parser.idb_version
        else:
            palisade_version = None
        test_log.events = Test.packets_to_events(test_log.packet_list,
                                                 palisade_version=palisade_version)

    # get output filepath
    xml_path = task.pipeline.get('test_log_xml',
                                 default=[None],
                                 args=True)[0]

    # if no file path, then generate it
    if xml_path is None:
        xml_path = test_log.file_name_format(
                task.pipeline,
                'solo_L0_rpw-mebgse-testlog',
                data_version=data_version,
                overwrite=overwrite) + '.xml'

    # Load GSE test log template
    template = JENV.get_template(TESTLOG_TEMPLATE)

    # Build the MEB GSE test log XML template render
    render = template.render(name=test_log.name,
                             uuid=test_log.uuid,
                             creation_date=test_log.creation_date.strftime(TESTLOG_STRFTIME),
                             author=getpass.getuser(),
                             description=test_log.description,
                             status=test_log.status,
                             launched=test_log.date.strftime(TESTLOG_STRFTIME),
                             terminated=test_log.terminated_date.strftime(TESTLOG_STRFTIME),
                             event_list=test_log.events,
                             )

    # Generate the XML
    # open a file with the name
    if os.path.isfile(xml_path) and not overwrite:
        logger.warning(f'{xml_path} already exists, aborting!')
    else:
        logger.info(f'Saving {xml_path}...')
        # Create output MEB GSE test log file
        with open(xml_path, 'w') as outfile:
            outfile.write(render)

    task.outputs['test_log_xml'].filepath = xml_path

@DryRunner.dry_run
@FileTarget.input(identifier='test_log_xml', filepath=get_test_log_xml)
@FileTarget.output(identifier='test_log_xml')
@Task.as_task(plugin_name='roc.guest', name='copy_xml')
def copy_xml(task):
    """
    Task to copy the input Test log LZ file into the output test directory
    with the ROC LZ filename convention (see ROC-TST-GSE-NTT-00017-LES doc.)
    (Use with from_xml command in dryrun mode only)

    :param task:
    :return:
    """

    # get the input target filepath
    input_testlog_xml = task.inputs['test_log_xml'].filepath

    # Create test log object
    test_log = Test.from_testlog_xml(input_testlog_xml)

    # Build the name of the output file
    output_testlog_xml = test_log.file_name_format(
        task,
        'solo_LZ_rpw-gse-test-log',
    ) + '.xml'

    # # Copy new test log into the output dir.
    logger.info('Copying {0} into {1}'.format(input_testlog_xml, output_testlog_xml))
    copy(input_testlog_xml, output_testlog_xml)
    if not os.path.isfile(output_testlog_xml):
        raise logger.error(
            'The test log file {0} has not been copied correctly in {1}!'.format(input_testlog_xml, output_testlog_xml))
    # else:
        #logger.debug("The test log file {0} has been copied correctly in {1}!".format(init_test_log, new_test_log))

    task.outputs['test_log_xml'].filepath = output_testlog_xml



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .base import Base
from sqlalchemy.ext.declarative import DeferredReflection
from sqlalchemy import Column, Integer, String

__all__ = ["FeedBackStatus"]


class FeedBackStatus(DeferredReflection, Base):
	__tablename__ = "feedbackstatus"

	ID = Column(Integer, primary_key=True)
	Status = Column(String)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""GUEST plugin tasks related to ADS GSE ASCII file handling."""

import sys
import os
import uuid

from edds_process.response.make import make_tmraw_xml, make_tcreport_xml

try:
    from poppy.core.logger import logger
    from poppy.core.db.connector import Connector
    from poppy.core.db.handlers import get_or_create_with_info
    from poppy.core.task import Task
    from poppy.core.target import FileTarget, PyObjectTarget
    from poppy.core.db.dry_runner import DryRunner
except:
    sys.exit("POPPy framework seems to not be installed properly!")


from roc.guest.guest import Test
from roc.guest.guest import parse_ads_file

__all__ = ['raw_to_txt', 'test_to_txt', 'txt_to_test']

# time format in the ADS GSE ASCII file (e.g. 2019-11-06T10:53:35.195Z)
ADS_STRFTIME = '%Y-%m-%dT%H:%M:%S.%f'

@PyObjectTarget.input(identifier='raw_data')
@FileTarget.output(identifier='ads_gse_txt')
@Task.as_task(plugin_name='roc.guest', name='raw_to_txt')
def raw_to_txt(task):
    # get the raw data from pipeline properties
    try:
        raw_data = task.inputs['raw_data'].value
    except:
        logger.error("No input RawData object defined, aborting!")
        return

    # Get output file path from input argument
    output_file = task.pipeline.args.ads_gse_txt

    logger.info(f'Writing {output_file}...')
    with open(output_file, 'w') as out_file:
        for current_packet in raw_data.packet_list:
            utc_time = current_packet['utc_time'].strftime(ADS_STRFTIME)[:-3] + 'Z'

            out_file.write(" ".join([utc_time,
                           current_packet['binary']]) + "\n")

    if os.path.isfile(output_file):
        logger.info("{0} saved".format(output_file))
    else:
        logger.error(f'Writing {output_file} has failed!')

    task.outputs['ads_gse_txt'].filepath = output_file

def get_ads_gse_txt(pipeline):
    """
    Get path of the ads gse txt file

    :param pipeline:
    :return:
    """
    return pipeline.args.ads_gse_txt


@DryRunner.dry_run
@FileTarget.input(identifier='ads_gse_txt', filepath=get_ads_gse_txt)
@PyObjectTarget.output(identifier='raw_data')
@Task.as_task(plugin_name='roc.guest', name='txt_to_test')
def txt_to_test(task):
    """
    Generate a Test instance from an input ADS GSE Text file.

    :param task:
    :return:
    """

    # Get --remove-header keyword value, if set
    try:
        header = task.pipeline.args.remove_header
    except:
        header = None

    # Parse ads file
    ads_gse_txt = task.inputs['ads_gse_txt'].filepath
    logger.info("Parsing {0}...".format(ads_gse_txt))
    raw_binary_packets = parse_ads_file(ads_gse_txt,
                                        remove_header=header)

    # Get packet times to compute min/max
    packet_times = [pkt['utc_time'] for pkt in raw_binary_packets]

    # Create test object from ADS text file...
    launched_date = min(packet_times)
    terminated_date = max(packet_times)
    test_log = Test(
        os.path.basename(os.path.splitext(ads_gse_txt)[0]),
        launched_date,
        test_uuid=str(uuid.uuid4()),
        creation_date=launched_date, # we assume that creation_date = launched_date
        description="Test created from {0}".format(ads_gse_txt),
        terminated_date=terminated_date,
        status='Terminated',
        file_path=ads_gse_txt,
    )

    # Add packet list to test object
    test_log.packet_list = raw_binary_packets

    logger.info(f'{len(raw_binary_packets)} packets retrieved from {ads_gse_txt}')

    # And insert it into the pipeline
    task.outputs['raw_data'].value = test_log

@FileTarget.input(identifier='txt_ads_file')
@FileTarget.output(identifier='dds_response_xml')
@Task.as_task(plugin_name='roc.guest', name='txt_to_dds')
def txt_to_dds(task):
    """
    Convert an input ADS GSE ASCII file into a MOC DDS response XML file.

    :param task:
    :return:
    """

    # Get --remove-header keyword value, if set
    try:
        header = task.pipeline.args.remove_header
    except:
        header = None

    # Get DDS XML filepath from output_file input argument
    try:
        output_file = task.pipeline.args.dds_response_xml
    except:
        logger.error('No output filename passed as argument, aborting')
        return None

    # Parse ads file
    ads_gse_txt = task.inputs['ads_gse_txt'].filepath
    logger.info("Parsing {0}...".format(ads_gse_txt))
    packet_times, raw_binary_packets = parse_ads_file(ads_gse_txt,
                                                     remove_header=header)
    # Generate output DDS XML
    # FIXME - Must filter between TM and TC data
    if make_tmraw_xml(raw_binary_packets, task.pipeline.properties.output_file,
                    overwrite=task.pipeline.properties.overwrite):
        logger.info("{0} saved".format(output_file))
    else:
        logger.error(f'Writing {output_file} has failed!' )

    # Set filepath value of output target
    task.outputs["dds_response_xml"].filepath = output_file

@DryRunner.dry_run
@PyObjectTarget.input(identifier='raw_data')
@FileTarget.output(identifier='ads_gse_txt')
@Task.as_task(plugin_name='roc.guest', name='test_to_txt')
def test_to_txt(task):
    """

    :param task:
    :return:
    """

    # Get test log raw data
    test_log = task.inputs['raw_data'].value

    # Get output file path
    output_file = task.pipeline.args.ads_gse_txt

    try:
        nevents = 0
        with open(output_file, "w") as file_out:
            # parse input test log XML and iterate on each EventDescr tag
            for event in test_log.events:
                time = "T".join([event["EventDate"], event["EventTime"]]) + "Z"
                row = " ".join([time, event["Data"]]) + "\n"
                file_out.write(row)
                logger.debug("Saving {0} into {1}".format(row, output_file))
                nevents += 1
    except Exception as e:
        logger.error(f'Writing {output_file} has failed: {e}')
    else:
        # events found
        if nevents == 0:
            logger.info("No event found in the input file, output file is empty!")
        else:
            logger.info("{0} event(s) saved into {1}".format(nevents, output_file))
    finally:
        task.outputs['ads_gse_txt'].filepath = output_file










#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .base import Base
from sqlalchemy.ext.declarative import DeferredReflection

__all__ = ["Device"]


class Device(DeferredReflection, Base):
    __tablename__ = "device"




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""GUEST plugin tasks related to MEB GSE database handling."""

import sys
from datetime import datetime
from threading import Thread
from queue import Queue

from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound
from sqlalchemy import and_

try:
    from poppy.core.logger import logger
    from poppy.core.db.connector import Connector
    from poppy.core.db.handlers import get_or_create_with_info
    from poppy.core.task import Task
    from poppy.core.target import FileTarget, PyObjectTarget
    from poppy.core.db.dry_runner import DryRunner
except:
    sys.exit('POPPy framework seems to not be installed properly, exiting!')


from roc.guest.models.test import TestLog

from roc.guest.models.meb_gse_data.test import Descriptor
from roc.guest.models.meb_gse_data.packet import PacketType
from roc.guest.models.meb_gse_data.parser import ParserCollection
from roc.guest.models.meb_gse_data.event import Event
from roc.guest.models.meb_gse_data.device import Device
from roc.guest.exceptions import MebDbTransactionError
from roc.guest.constants import TESTLOG_STRFTIME
from roc.guest.guest import Test



__all__ = ['test_to_mebdb', 'mebdb_to_test']

class mebdb_to_test(Task):

    plugin_name = 'roc.guest'
    name = 'mebdb_to_test'

    def add_targets(self):
        self.add_output(target_class=PyObjectTarget,
                        identifier='test_descriptors')

    def set_filters(self):

        # If --Terminated option is set, only get Terminated tests
        if ('terminated' in self.pipeline.args and
                self.pipeline.args.Terminated is True):
            self.terminated = True
        else:
            self.terminated = False

        # If pattern option is set, retrieve
        # only tests that contain the pattern string
        if ('pattern' in self.pipeline.args and
                self.pipeline.args.pattern is not None):
            self.pattern = self.pipeline.args.pattern[0]
        else:
            self.pattern = None

        # If --time-max option is set, add an upper time limit
        if ('time_max' in self.pipeline.args and
                self.pipeline.args.time_max is not None):
            self.time_max = datetime.strptime(
                self.pipeline.args.time_max[0], TESTLOG_STRFTIME)
        else:
            self.time_max = None

        # If --time-min option is set, add an lower time limit
        if ('time_min' in self.pipeline.args and
                self.pipeline.args.time_min is not None):
            self.time_min = datetime.strptime(
                self.pipeline.args.time_min[0], TESTLOG_STRFTIME)
        else:
            self.time_min = None

    def run(self):

        """
        Task to generate an output XML file containing
        test log data retrieved from the MEB GSE database.

        :param task:
        :return:
        """

        # Set input filters
        self.set_filters()

        # Extract TestDescriptor of the requested test log data
        self.test_descriptors = self.extract_descriptors()

        # Set output target value
        self.outputs['test_descriptors'].value = self.test_descriptors


    @Connector.if_connected('MEB')
    def extract_descriptors(self):
        """
        Extract tests from the MEB GSE database.

        :param time_min: Get only test data after time_min (datetime object)
        :param time_max: Get only test data before time_max (datetime object)
        :param pattern: Get only test data with input pattern (MySQL regex string)
        :param terminated: if True only get test data for terminated test
        :return: Descriptor object filled with requested test data
        """

        # get the meb connector and get the database
        connector = Connector.manager['MEB']
        database = connector.get_database()

        # check the database is connected
        database.connectDatabase()

        # get the selected database from the MEB connector
        self.meb = connector.selected

        # connect to the database
        self.meb.connectDatabase()

        # check that the connection is made
        if not self.meb.connected:
            message = '{0} is not connected'.format(self.meb)
            logger.error(message)
            raise MebDbTransactionError(message)

        # Add input filters
        filters = []
        # If --Terminated option is set, only get Terminated tests
        if self.terminated == True:
            filters.append(Descriptor.Status == 'Terminated')

        # If pattern option is set, retrieve
        # only tests that contain the pattern string
        if self.pattern:
            filters.append(Descriptor.TestName.like(self.pattern))

        # If --time-max option is set, add an upper time limit
        if self.time_max:
            filters.append(Descriptor.TerminatedDate <= self.time_max)

        # If --time-min option is set, add an lower time limit
        if self.time_min:
            filters.append(Descriptor.Launched >= self.time_min)

        # get a session
        with self.meb.query_context() as session:
            # get list of tests in MEB GSE database
            try:
                return session.query(
                    Descriptor.TestName,
                    Descriptor.UUID,
                    Descriptor.XML,
                    Descriptor.Launched,
                    Descriptor.TerminatedDate,
                    Descriptor.Status,
                    Descriptor.CreationDate,
                    Descriptor.Description,
                    Descriptor.Author,
                ).filter(and_(*filters))
            except NoResultFound:
                logger.info('MEB GSE database query has returned no result!')
                return []

    def generator(self, loop):
        """Test log generator."""

        # reference to the pipeline
        pipeline = loop.pipeline

        # reference to the start task
        start_task = loop.start

        # reference to the end task
        end_task = loop.end

        # loop over tests into the MEB GSE database
        nextracted = 0

        for test_desc in loop.inputs['test_descriptors']:

            # check that the test is already present in the ROC database or not
            with self.meb.query_context() as session:
                if session.query(FileLog).filter_by(
                    file_id=test_desc.value.UUID,
                    file_name=test_desc.value.TestName,
                    is_test=True,
                ).count():
                    logger.info('{0} already loaded'.format(test_desc.value.TestName))
                    continue

            # set the test to parse
            logger.info('Extracting test {0}'.format(test_desc.value.TestName))
            self.test_descriptor = test_desc.value
            try:
                self.extract_events()
            except Exception as e:
                logger.error(f'Test {test_desc.value.TestName} ({test_desc.value.UUID}) has not event, skip it')
                continue
            else:
                test_desc.value = mebdb_to_test.to_test(self.test_descriptor, self.test_events)
                nextracted += 1

            # replace the 'raw_data' target by the test_desc instance
            test_desc.link('raw_data')

            yield

        if nextracted == 0:
            logger.info('All requested tests have been already extracted!')
            sys.exit(1)

    @Connector.if_connected('MEB')
    def extract_events(self):
        """
        A simple method that convert a test log from the
        MEB GSE database into an XML file.
        """

        # Initialize output test events list
        self.test_events = []

        if not self.test_descriptor:
            logger.error('No input test_descriptor defined!')
            return []

        # number of events
        logger.debug(
            'Counting number of events for test {0}'.format(
                self.test_descriptor.TestName
            )
        )
        with self.meb.query_context() as session:
            query = session.query(Event.XML).join(Descriptor).filter(
                Descriptor.TestName == self.test_descriptor.TestName and
                Descriptor.UUID == self.test_descriptor.UUID
            )
            nevents = query.count()
            logger.info(
                '{0} test events to retrieve'.format(nevents)
            )

        # create a queue to transfer data
        queue = Queue()

        # create a thread to request the data regularly from the database
        thread = Thread(
            target=self.worker,
            args=[self, queue, 10000, nevents],
        )

        # start the thread
        thread.start()

        # loop while there is data
        offset = 0
        event_list = []
        while (offset < nevents):
            # get the offset and the events
            logger.debug('Waiting data from thread')
            step, data = queue.get()
            logger.debug('Process events from thread')

            # increase offset
            offset = step

            # loop over events to add them into the xml
            for event in data:
                # read the xml in the string
                try:
                    event_list.append(event.XML)
                except:
                    logger.error("Can't convert test event into XML")
                    return []

            logger.debug('Events processed')

        # block until end
        thread.join()

        self.test_events = event_list

    @staticmethod
    def to_test(test_descriptor, test_events):
        """
        Convert input test_descriptor and test_events data
        into a Test class instance

        :return: Test class instance containing test_descriptor/test_events data
        """

        # Create Test instance with info in test_descriptor
        launched_date = datetime.strptime(test_descriptor.Launched, TESTLOG_STRFTIME)
        terminated_date = datetime.strptime(test_descriptor.TerminatedDate, TESTLOG_STRFTIME)
        creation_date = datetime.strptime(test_descriptor.CreationDate, TESTLOG_STRFTIME)

        test_instance = Test(
            test_descriptor.TestName, launched_date,
            test_uuid=test_descriptor.UUID,
            creation_date=creation_date,
            description=test_descriptor.Description,
            terminated_date=terminated_date,
            status=test_descriptor.Status,
            author=test_descriptor.Author,
        )

        # Generate the list of test events
        test_instance.events = [test_event for test_event in test_instance.events_from_string(test_events)]

        # return test instance
        return test_instance

    @staticmethod
    def worker(self, queue, step, nmax):
        """
        Worker in an other thread, to get the events by windows, in order to
        process it in the same time.
        """
        # init the offset
        offset = 0

        # loop while not reaching the end
        while (offset < nmax):
            # make the query to get events associated to the test. Do not use
            # the relationship of sqlalchemy since it keeps in memory the
            # attribute of events and doesn't allow it to be removed, else a
            # DELETE statement is performed on the database
            with self.meb.query_context() as session:
                query = session.query(Event.XML).join(Descriptor).filter(
                    Descriptor.TestName == self.test_descriptor.TestName and
                    Descriptor.UUID == self.test_descriptor.uuid
                )
                # query events in the window
                logger.debug('Get {0} events maximum from thread'.format(step))
                events = query.limit(step).offset(offset).all()

            # increase the offset
            offset += step

            # put the data in the queue
            logger.debug('Sending events to the queue')
            queue.put((offset, events))

@PyObjectTarget.output(identifier='raw_data')
@Task.as_task(plugin_name='roc.guest', name='test_to_mebdb')
@Connector.if_connected('MEB')
def test_to_mebdb(task):
    """
    Insert a test log object data into a MEB GSE database

    :param test_log: Test object to insert
    :return: True if correctly inserted, False otherwise
    """

    # Get input value
    test_log = task.outputs['raw_data'].value

    # get the meb connector and get the database
    connector = Connector.manager['MEB']
    database = connector.get_database()

    # check the database is connected
    database.connectDatabase()

    # get the selected database from the MEB connector
    meb = connector.selected

    # connect to the database
    meb.connectDatabase()

    # check that the connection is made
    if not meb.connected:
        message = '{0} is not connected'.format(meb)
        logger.error(message)
        raise MebDbTransactionError(message)

    # Insert test log TestDescriptor data into the MEB GSE database
    logger.info(f'Inserting test log [{test_log.name}, {test_log.uuid[:7]}] into {meb} ({len(test_log.events)} events to insert) ...')
    with meb.query_context() as session:

        descriptor, is_created = get_or_create_with_info(
            session,
            Descriptor,
            TestName=test_log.name,
            Status=test_log.status,
            Author=test_log.author,
            Description=test_log.description,
            CreationDate=test_log.creation_date,
            Launched=test_log.date,
            TerminatedDate=test_log.terminated_date,
            UUID=test_log.uuid,
            XML=test_log.descriptor_xml,
            OneShot=1
        )

    # If already found, then aborting
    if is_created == False:
        logger.info(f'Test {test_log.uuid} already found in the database')
        return False

    # Loop over test events
    logger.debug(f'test log description inserted')
    logger.debug(f'Inserting test event data ...')
    for event in test_log.events:

        with meb.query_context() as session:

            # Get all needed ID from MEB db

            query_collection = session.query(ParserCollection).filter(
                ParserCollection.CollectionName == event['Collection']
            )
            query_event = session.query(PacketType).filter(
                PacketType.Type == event['EventType']
            )
            query_desc = session.query(Descriptor).filter(
                Descriptor.TestName == test_log.name
            )
            query_device = session.query(Device).filter(
                Device.DeviceName == event['Device']
            )

            dic = [
                {'item_name': 'collection_id', 'query': query_collection, 'item_value' : event['Collection'] },
                {'item_name': 'event_id', 'query': query_event,'item_value': event['EventType']},
                {'item_name': 'descriptor_id', 'query': query_desc,'item_value' : test_log.name },
                {'item_name': 'device_id', 'query': query_device,'item_value' :event['Device'] }
            ]

            res = {}
            for item in dic:
                val = item['item_value']
                name = item['item_name']
                try:
                    res[name] = int(item['query'].one().ID)
                except NoResultFound:
                    logger.error(f'No {name} found for {val} in the MEB GSE database!')
                    break
                except MultipleResultsFound:
                    logger.error(f'More than one {name} found for { val } in the MEB GSE database!')
                    break

            # Insert test log event data into MEB GSE database
            event, is_created = get_or_create_with_info(
                session,
                Event,
                DateProduction=event['EventDate'] + 'T' + event['EventTime'],
                XML=event['XML'],
                Device=res['device_id'],
                PacketType=res['event_id'],
                AssociatedTest=res['descriptor_id'],
                Category=event['Category'],
                Name=event['Name'],
                Collection=res['collection_id'],
            )
            if is_created == False:
                logger.error(f'Current event not inserted: ({event})')
                logger.debug(
                    DateProduction=event['EventDate'] + 'T' + event['EventTime'],
                    XML=event['XML'],
                    Device=res['device_id'],
                    PacketType=res['event_id'],
                    AssociatedTest=res['descriptor_id'],
                    Category=event['Category'],
                    Name=event['Name'],
                    Collection=res['collection_id']
                )
                continue

    logger.debug(f'Test event data inserted')
    return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from poppy.core.logger import logger

from functools import lru_cache
from poppy.core.db.connector import Connector
from poppy.core.generic.metaclasses import Singleton
from poppy.core.db.database import Database
from .models.meb_gse.database import DatabaseInformation

__all__ = ["MEB", "MEBException"]




class MEBException(Exception):
    pass


class MEBSelector(object, metaclass=Singleton):
    """
    Class to manage the database of the MEB which are *backuped* regularly but
    must be available to the user.
    """
    def __init__(self, meb):
        """
        Need the instance of the MEB database as an argument to be able to
        query the other provided databases.
        """
        self.meb = meb

    def current(self):
        """
        Return the database marked as current.
        """
        # check connected and reflected
        if not self.meb.is_available():
            logger.error("{0} is not connected".format(self.meb))
            return None

        # get a session from the database
        with self.meb.query_context() as session:

            # get the current database
            tmp_meb = session.query(DatabaseInformation).filter(
                DatabaseInformation.Current==1
            )
            meb = tmp_meb.one().DatabaseName

        # check that there is a current database
        if meb is None:
            message = "No current database for MEB. Why?"
            logger.error(message)
            raise MEBException(message)

        # generate the database object for the current
        return self.select(meb)

    @lru_cache(maxsize=32)
    def select(self, name):
        """
        Return a database according to its name.
        """
        # make a copy of parameters of the database for common informations
        parameters = {k: v for k, v in self.meb.parameters.items()}

        # update parameters with the information on the name of the meb
        # database FIXME: here forced for purpose tests
        parameters["database"] = name

        # create the database
        meb = Database(name)
        meb.parameters = parameters
        meb.base = Database.bases_manager.get("MEB")

        # return the database
        return meb


class MEB(Connector):
    """
    The connector class for the MEB database.
    """
    def __init__(self, *args, **kwargs):
        """
        Init the selected database to None.
        """
        # create instance as usual
        super(MEB, self).__init__(*args, **kwargs)

        # indicate that no database is selected
        self._selected = None

    @property
    def database(self):
        return self._database

    @database.setter
    def database(self, name):
        self._database = name

        # also set the database to the selector
        self.selector = MEBSelector(self.get_database())

    def select(self, name):
        """
        To select the MEB database for the data that will be used for the other
        selections on the database. Thus the database will be the current one
        in internal to do tasks.
        """
        # get the database from the selector
        self.selected = self.selector.select(name)

        # return the selected database
        return self.selected

    @property
    def selected(self):
        """
        Get the current database from the MEB informations if no one is already
        selected.
        """
        if self._selected is None:
            self._selected = self.selector.current()
        return self._selected

    @selected.setter
    def selected(self, value):
        self._selected = value




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .base import Base
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import DeferredReflection

__all__ = ["Event"]


class Event(DeferredReflection, Base):
    __tablename__ = "eventdescr"

    device = relationship("Device")
    collection = relationship("ParserCollection")
    test = relationship("Descriptor")
    packet_type = relationship("PacketType")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .base import Base
from sqlalchemy.ext.declarative import DeferredReflection
from sqlalchemy import Column, Integer, String

__all__ = ["ParserCollection"]


class ParserCollection(Base):
    __tablename__ = "parsercollection"

    ID = Column(Integer, primary_key=True)
    CollectionName = Column(String)





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import xml.etree.cElementTree as ET
import os.path as osp
import datetime as dt
import uuid
import os
import hashlib
from pathlib import Path

from sqlalchemy.orm.exc import MultipleResultsFound
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy import func

from poppy.core.logger import logger

from poppy.core.tools.text import get_valid_filename
from poppy.core.db.connector import Connector
from poppy.core.generic.cache import CachedProperty
from poppy.core.tools.hashfile import sha256_from_file

from roc.rpl.packet_parser import RawData
from roc.rpl.packet_parser.palisade import palisade_metadata
from roc.film.tools import valid_data_version

from roc.guest.models.test import TestLog
from roc.guest.constants import TIME_RANGE_STRFORMAT, \
    INVALID_ADS_TIME, INPUT_TIME_STRFORMAT, TESTLOG_STRFTIME, \
    TESTLOG_EVENTTIME_STRFORMAT, TESTLOG_EVENTDATE_STRFORMAT, TIME_ISO_STRFORMAT
from roc.guest.exceptions import ParseTestXmlError

__all__ = ['Test', 'parse_ads_file',
           'create_file_log']

class Request(object):
    """
    Container for request information, to be able to share it across tasks.
    """
    def __init__(self, sha256):
        """
        Create a request object from the necessary, mandatory minimal information
        from the point of view of the ROC.
        """
        # store minimal information
        self.sha256 = sha256
        self.uuid = uuid.UUID(sha256[:32])

    @classmethod
    def from_test_log_xml(cls, xml_path):
        # compute the uuid from file content
        sha256 = sha256_from_file(xml_path)
        return cls(sha256)


class Test(RawData):
    """
    Container for information on a test and be able to share it across tasks.
    """
    def __init__(self, test_name, test_date,
                 test_uuid=str(uuid.uuid4()),
                 creation_date=dt.datetime.today(),
                 description=None,
                 terminated_date=dt.datetime.today(),
                 status='Terminated',
                 author=None,
                 events=[],
                 file_path=None,
                 version=None):
        """
        Create a test object from the necessary, mandatory minimal information
        from the point of view of the ROC.

        :param test_name: Name of the test
        :param test_date: Launch date of the test
        :param test_uuid: UUID of the test
        :param creation_date: Creation date of the test
        :param description: Description of the test
        :param terminated_date: End date of the test
        :param status: Status of the test
        :param events: List of events inside the test
        :param file_path: Path to the test log file
        :param version: Version of the data (if any)
        """

        # Init RawData class
        super(Test, self).__init__()

        # store minimal information
        if test_name:
            self.name = test_name
        else:
            logger.warning(f'No test_name for input data, use test_uuid instead: {test_uuid}')
            self.name = test_uuid

        if test_date:
            self.date = test_date
        else:
            self.date = dt.datetime.today()
            logger.error(f'No test_date for input data, use {self.date} instead!')

        self.date = test_date
        self.uuid = test_uuid
        self.status = status
        self.creation_date = creation_date
        self.terminated_date = terminated_date
        self.description = description
        self.author = author
        self.file_path = file_path
        self.events = events
        self.version = version

        # create an identity
        self.sha1 = self.test_sha1()


    @property
    def time_min(self):
        return self.date

    @property
    def time_max(self):
        return self.terminated_date

    @property
    def descriptor_xml(self):
        return Test.build_descriptor_xml(self)

    @staticmethod
    def build_descriptor_xml(test):
        """
        Build TestDescriptor XML

        :return: TestDescriptor XML
        """

        if isinstance(test.terminated_date, dt.datetime):
            terminated_date = test.terminated_date.strftime(TIME_RANGE_STRFORMAT)
        elif isinstance(test.terminated_date, str):
            terminated_date = test.terminated_date
        else:
            terminated_date = ''
            logger.warning(f'Wrong terminated_date type ({type(terminated_date)})')

        xml = '<TestDescriptor'
        xml += f' TestName="{test.name}"'
        xml += f' TestUUID="{test.uuid}">'
        xml += '<GeneralInfo>'
        xml += f'<CreationDate>{test.creation_date.strftime(TIME_RANGE_STRFORMAT)}</CreationDate>'
        xml += f'<LongDescription>{test.description}</LongDescription>'
        xml += '</GeneralInfo>'
        xml += '<ExecutionInformation>'
        xml += f'<Status>{test.status}</Status>'
        xml += f'<LaunchedDate>{test.date.strftime(TIME_RANGE_STRFORMAT)}</LaunchedDate>'
        xml += f'<TerminatedDate>{terminated_date}</TerminatedDate>'
        xml += '</ExecutionInformation>'
        xml += '</TestDescriptor>'

        return xml

    @classmethod
    def from_meb_test(cls, test):
        """
        Create a test object for the information contained in the test of the
        MEB-GSE.
        """
        return cls(test.TestName, test.Launched, test_uuid=test.UUID)

    @classmethod
    def from_roc_test(cls, test):
        """
        Create a test object from information in the ROC database.
        """
        return cls(test.test_name, test.test_launched_date, test_uuid=test.test_uuid)

    @classmethod
    def from_testlog_xml(cls, xml_path,
                         header_only=False):
        """
        Parse the MEB GSE test log XML file and use the information
        to generate an instance of the Test class.

        :param xml_path: Path to the MEB GSE test log XML file
        :param header_only: If True, then retrieve test log header info only
        :return: instance of the Test class corresponding to the input test log XML information
        """

        logger.info(f'Parsing {xml_path} ...')

        test_name, test_uuid, launch_date, terminated_date, \
        creation_date, description, status, test_author = cls.get_testlog_header(xml_path)

        if not header_only:
            # Retrieve eventdescr too
            events = [event
                      for event in cls.events_from_file(xml_path)
                      ]
        else:
            events = []

        # return the test
        return cls(test_name, launch_date,
                   test_uuid=test_uuid,
                   terminated_date=terminated_date,
                   creation_date=creation_date,
                   description=description,
                   status=status,
                   author=test_author,
                   file_path=xml_path,
                   events=events)


    @staticmethod
    def get_testlog_header(xml_path):
        """
        Extract test log header info from the input file.

        :param xml_path: path of the MEB GSE test log XML file
        :return: test_name, test_uuid, launch_date, terminated_date, creation_date, description, status
        """

        # Make sure to have a string and not a list
        if isinstance(xml_path, list):
            xml_path = xml_path[0]

        # open the file
        try:
            with open(xml_path, 'r') as f:
                # loop without parsing on the document
                for event, elem in ET.iterparse(f, ['end']):
                    # if this an TestDescriptor
                    if elem.tag == 'TestDescriptor':
                        # store information about the test
                        test_name = elem.attrib['TestName']
                        test_uuid = elem.attrib['TestUUID']

                        # one for general information
                        gen_info = elem.find('GeneralInfo')
                        description = gen_info.find('LongDescription').text
                        author_tag = gen_info.find('Author')
                        if author_tag:
                            test_author = author_tag.text
                        else:
                            test_author = 'Undefined'
                        creation_date = dt.datetime.strptime(
                            gen_info.find(
                                'CreationDate'
                            ).text,
                            '%Y-%m-%dT%H:%M:%S',
                        )

                        # one for execution information
                        exec_info = elem.find('ExecutionInformation')
                        status = exec_info.find('Status').text
                        launch_date = dt.datetime.strptime(
                            exec_info.find(
                                'LaunchedDate'
                            ).text,
                            '%Y-%m-%dT%H:%M:%S',
                        )
                        terminated_date = dt.datetime.strptime(
                            exec_info.find(
                                'TerminatedDate'
                            ).text,
                            '%Y-%m-%dT%H:%M:%S',
                        )
        except Exception as e:
            logger.error(f'Parsing {xml_path} has failed: {e}')
            raise ParseTestXmlError

        return test_name, test_uuid, launch_date, terminated_date, \
               creation_date, description, status, test_author

    @staticmethod
    def events_from_file(xml_path):
        """
        Retrieve eventdescr data from test log XML, iterating on each tag found with yield command.

        :param xml: path of the MEB GSE test log xml file
        :return: dictionary containing the eventdescr info
        """

        with open(xml_path, 'r') as buff:
            # loop without parsing on the document
            for event, elem in ET.iterparse(buff, ['end']):
                # if this an TestDescriptor
                eventdescr = Test.xml_to_event(elem)
                if eventdescr:
                    yield eventdescr
                else:
                    continue

    @staticmethod
    def events_from_string(xml_string):
        """
        Retrieve eventdescr data from XML, iterating on each tag found with yield command.

        :param xml_string: List of XML string containing the eventdescr data
        :return: dictionary containing the eventdescr info
        """

        # Make sure that xml_string input is a list
        if not isinstance(xml_string, list):
            xml_string = [xml_string]

        # Loop over xml string list to get EventDescr tag data
        # and return it as a Test event
        for xml in xml_string:
            yield Test.xml_to_event(ET.fromstring(xml))

    @staticmethod
    def xml_to_event(xml_elem):
        """
        Store a xml element as a eventdescr dictionary

        :param xml_elem: xml element to parse
        :return: dictionary with eventdescr data
        """

        eventdescr = None
        if xml_elem.tag == 'EventDescr':
            # Only extract S_C device tags
            if (xml_elem.attrib['Device'] == 'S_C' and
                    (xml_elem.attrib['EventType'] == 'TM' or
                     xml_elem.attrib['EventType'] == 'TC_FEEDBACK')):
                eventdescr = {'EventDate': xml_elem.find('Common').find('EventDate').text,
                              'EventTime': xml_elem.find('Common').find('EventTime').text,
                              'Name': xml_elem.find('Content').find('Name').text,
                              'Data': xml_elem.find('Content').find('Data').text,
                              'Device': 'S_C',
                              'TestUUID': xml_elem.find('Common').find('TestUUID').text,
                              'EventType': xml_elem.attrib['EventType'],
                              'EventID': xml_elem.attrib['EventID'],
                              'Category': xml_elem.find('Content').find('Category').text,
                              'Collection': xml_elem.find('Content').find('Collection').text,
                              }

                # Get TC feedback status
                feedback = xml_elem.find('FeedBack')
                if feedback:
                    eventdescr['Status'] = feedback.find('Status').text

                # Build eventdescr xml
                eventdescr_xml = Test.event_to_xml(eventdescr)
                eventdescr['XML'] = eventdescr_xml

        return eventdescr

    @staticmethod
    def event_to_xml(eventdescr):
        """
        Build EventDescr xml from an input eventdescr dictionary

        :param eventdescr: dictionary containing event tags/attributes
        :return: xml of eventdesc
        """

        xml = f'<EventDescr Device="{eventdescr["Device"]}"'
        xml += f' EventID="{eventdescr["EventID"]}"'
        xml += f' EventType="{eventdescr["EventType"]}">'
        xml += '<Common>'
        xml += f'<EventDate>{eventdescr["EventDate"]}</EventDate>'
        xml += f'<EventTime>{eventdescr["EventTime"]}</EventTime>'
        xml += f'<TestUUID>{eventdescr["TestUUID"]}</TestUUID>'
        xml += '</Common>'
        xml += '<Content>'
        xml += f'<Name>{eventdescr["Name"]}</Name>'
        xml += f'<Category>{eventdescr["Category"]}</Category>'
        xml += f'<Collection>{eventdescr["Collection"]}</Collection>'
        xml += f'<Data>{eventdescr["Data"]}</Data>'
        xml += '</Content>'
        xml += '</EventDescr>'

        return xml

    @CachedProperty
    def short_sha1(self):
        return self.sha1[:7]
        #  return self.get_short_test_id()

    def test_sha1(self):
        """
        Given the characteristics of a test from the point of view of the ROC
        (name, uuid and date), gives the sha1 of the test that will be used to
        refer as an identity of the test.
        """
        # create sha1 object
        sha1 = hashlib.sha1()

        # update the object with information
        sha1.update(self.name.encode())
        sha1.update(self.uuid.encode())
        sha1.update(self.date.strftime(TESTLOG_STRFTIME).encode())

        # create the digest of the hash (the sha1)
        return sha1.hexdigest()

    def get_short_test_id(self, short=6, session=None):
        """
        Given the SHA-1 of a given test, search in the ROC database for a test
        with this already defined ID shorted to the length of the short
        argument. If this short SHA-1 is not unique, do the same thing with
        short incremented by one character.
        """

        # check the size of the short compared to test id
        if len(self.sha1) <= short:
            raise ValueError(
                (
                    'The test ID {0} with length {1} is too small for ' +
                    'shortening it to size {2}'
                ).format(self.sha1, len(self.sha1), short)
            )

        # get the connector of the roc to create a session to query the database
        if session is None:
            session = Connector.manager['MAIN-DB'].get_database().session()

        # query the ROC database for a test ID short as this one
        query = session.query(func.count(TestLog.test_sha))
        query = query.filter(
            TestLog.test_sha.like(self.sha1[:short] + '%')
        )

        # run the query
        try:
            # get number of test matching this short id
            count = query.one()[0]

            # if no one found, we can use the short id
            if count == 0:
                return self.sha1[:short]
            else:
                # run the query again but incrementing the short length
                self.get_short_test_id(short=short + 1, session=session)
        except NoResultFound:
            raise ValueError(
                (
                    'No result found when querying the count of SHA-1 ' +
                    'matching {0}'
                ).format(self.sha1[:short])
            )
        except MultipleResultsFound:
            raise ValueError(
                (
                    'Multiple results found when querying count of SHA-1 ' +
                    'matching {0}'
                ).format(self.sha1[:short])
            )

    def output_directory(self, task):
        """
        Given the pipeline, the name of a test and the ID of the test, create a
        directory for the data output of the test and validate it.

        :param task: Task object
        :return: string containing output directory path
        """
        # get the valid file name of the test, like slug for URL
        valid_filename = get_valid_filename(self.name)

        # get the short test ID
        short_id = self.short_sha1

        # directory where to put results
        path = osp.join(
            task.output,
            '__'.join([valid_filename, short_id, task.provider]),
        )

        # create the directory if not already done
        os.makedirs(path, exist_ok=True)

        # return the path
        return path

    def file_name_format(self, task, prefix,
                         data_version=None,
                         overwrite=False):
        """
        Generate a file name following the ROC convention given the information
        on the pipeline, dataset, etc.

        :param task: Task object
        :param prefix: filename prefix
        :param data_version: version of the output file
        :param overwrite: overwrite any existing output file
        :return: path and name of the output test log file with the right convention.
        """

        # get output directory
        output = self.output_directory(task)

        # pre-process data_version
        if not data_version and self.version:
            data_version = self.version
        elif not data_version and not self.version:
            data_version = '01'

        # generate datetime and free_field
        datetime = '-'.join([self.date.strftime(TIME_RANGE_STRFORMAT),
                             self.terminated_date.strftime(TIME_RANGE_STRFORMAT)])
        free_field = task.provider[:3] + '-' + self.short_sha1

        filepath = self._generate_filepath(output, prefix, datetime, data_version, free_field)

        # If overwrite then replace existing file
        if osp.isfile(filepath) and overwrite:
            return filepath

        # Then increment version to get new file
        while osp.isfile(filepath):
            data_version = f'{int(data_version)+1:02d}'
            filepath = self._generate_filepath(output, prefix, datetime, data_version, free_field)

        return filepath

    @staticmethod
    def _generate_filepath(output, prefix, datetime, version, free_field):
        """
        Generate filepath of test log

        :param output: path of the output directory
        :param prefix: filename prefix
        :param datetime: filename time range
        :param version: filename version
        :param free_field: filename free field
        :return: filepath of test log
        """
        version = 'V' + valid_data_version(version)
        return os.path.join(
            output,
            '_'.join(
                [prefix, datetime, version, free_field.lower()]
            )
        )


    def to_testlog_repr(self):
        """
        Return Test object as a TestLog database representation dictionary .

        :param test_log: Test object to insert
        :param xml: string containing test log file
        :param descriptor: test descriptor XML
        :return: dictionary containing TestLog representation
        """

        return dict(
            file_parent=os.path.basename(self.file_path),
            test_name=self.name,
            test_id=self.uuid,
            test_creation_date=self.creation_date,
            test_descr=self.description,
            test_author=self.author,
            test_status=self.status,
            test_launched=self.date,
            test_terminated=self.terminated_date,
            test_insert_date=dt.datetime.today().strftime(TESTLOG_STRFTIME),
        )

    @staticmethod
    def event_to_packet_log(event, file_log_id):
        """
        Generate a packet log representation of the ROC database from a given eventdescr dictionary.

        :param event: eventdescr to convert
        :param file_log_id: index of the corresponding file_log entry
        :return: packet log dictionary
        """


        # Return a
        return dict(
                packet_length = event['Data'],
                packet_type = event['EventType'][:2],
                packet_category = event['Category'],
                packet_creation_time = None,
                sync_flag = None,
                utc_time = event['EventDate'] + 'T' + event['EventTime'],
                name = None,
                palisade_id = event['Name'],
                raw_data = event['Data'],
                ack_exe_state = None,
                ack_acc_state = None,
                sequence_name = None,
                unique_id = None,
                file_log_id = file_log_id,
                )


    @staticmethod
    def packet_to_packet_log(packet, file_log_id):
        """
        Generate a packet log representation of the ROC database from a given packet dictionary.

        :param packet: packet dictionary to convert
        :param file_log_id: index of the corresponding file_log entry
        :return: packet log dictionary
        """

        data_field_header = packet.get('data_field_header', None)
        if data_field_header:
            packet_creation_time = str(packet['data_field_header'].time[0]) + ':' + str(packet['data_field_header'].time[1])
            sync_flag = packet['data_field_header'].time[2] == 0
        else:
            packet_creation_time = None
            sync_flag = None

        # Return a dictionary with expected fields for packet log representation
        return dict(
                packet_length = packet['header'].packet_length,
                packet_type = packet['type'],
                packet_category = packet['category'],
                packet_creation_time = packet_creation_time,
                sync_flag = sync_flag,
                utc_time = packet['utc_time'],
                name = packet['srdb_id'],
                palisade_id = packet['palisade_id'],
                raw_data = packet['binary'],
                ack_exe_state = packet.get('ack_exe_state', None),
                ack_acc_state = packet.get('ack_acc_state', None),
                sequence_name = packet.get('sequence_name', None),
                unique_id = packet.get('unique_id', None),
                file_log_id = file_log_id,
                )

    @staticmethod
    def packets_to_events(packet_list,
                            collection='IDB',
                            device='S_C',
                            palisade_version=None):
        """
        Build list of EventDescr from input packets list.

        :param packet_list: list of packet data dictionaries
        :param collection: Test event collection.
        :param device: Test event device.
        :param palisade_version: string containing the version of PALISADE IDB
        :return: events a list of dictionaries with expected eventdescr tag values
        """

        # Initialize output list
        nevent = len(packet_list)
        events = [None] * nevent

        # Get PALISADE metadata for current packet
        palisade_metadata_dict = palisade_metadata(
            palisade_version=palisade_version)

        # Loop over each packet
        nvalid = 0
        for i, current_packet in enumerate(packet_list):

            packet_name = current_packet['srdb_id']
            packet_type = current_packet['type']

            if not packet_name:
                logger.warning('#{0} - Unknown packet, skipping!'.format(packet_name))
                continue
            else:
                event_date = current_packet['utc_time'].strftime(TESTLOG_EVENTDATE_STRFORMAT)
                event_time = current_packet['utc_time'].strftime(TESTLOG_EVENTTIME_STRFORMAT)[:-3]
                if packet_type == 'TC':
                    type = 'TC_FEEDBACK'
                else:
                    type = 'TM'

                event = {'id':'0',
                         'EventType':type,
                         'Name':palisade_metadata_dict[packet_name]['palisade_id'],
                         'Category':palisade_metadata_dict[packet_name]['packet_category'],
                         'Data':current_packet['binary'],
                         'EventDate':event_date,
                         'EventTime':event_time,
                         'Collection':collection,
                         'Device': device,
                }

                if packet_type == 'TC':
                    event['Status'] = current_packet['ack_exe_state']

                events[i] = event
                nvalid += 1

        if nvalid < nevent:
            logger.warning(f'{nevent - nvalid} event(s) are not valid!')

        return events[0:nvalid]

    @classmethod
    def from_l0(cls, l0_path):
        """
        Parse input RPW L0 hd5 file and use the information
        to generate an instance of the Test class.

        :param l0_path:
        :param header_only:
        :return:
        """

        # Import L0 class to get list of packets
        from roc.film.tasks.l0 import L0

        l0_basename = os.path.basename(l0_path)
        test_name = os.path.splitext(l0_basename)[0]

        #Get L0 metadata
        l0_header = L0.extract_header(l0_path)

        data_version = l0_header['Data_version']
        try:
            test_uuid = l0_header['Test_uuid']
            description = l0_header['Test_description']
            launch_date = l0_header['Test_launched_date']
            terminated_date = l0_header['Test_terminated_date']
        except:
            logger.warning(f'No "Test_*" attribute found in {l0_path}')
            test_uuid = l0_header['File_ID']
            description = f'Generated by RGTS from {l0_basename}'
            launch_date = l0_header['TIME_MIN']
            terminated_date = l0_header['TIME_MAX']

        # Make sure that times are datetime.datetime object
        launch_date = dt.datetime.strptime(launch_date,
                                  TIME_ISO_STRFORMAT)
        terminated_date = dt.datetime.strptime(terminated_date,
                                      TIME_ISO_STRFORMAT)

        # Get list of packets in the input L0 file
        logger.info(f'Parsing {l0_path} ...')
        l0_packet_list = L0.l0_to_packet_list([l0_path],
                               no_data=True,
                               no_header=True,
                               ascending=True)

        events = [Test.l0_packet_to_event(current_packet, test_uuid)
                  for current_packet in l0_packet_list
                  if L0.is_valid_packet(current_packet)]

        # return the test
        return cls(test_name, launch_date,
                   test_uuid=test_uuid,
                   terminated_date=terminated_date,
                   description=description,
                   events=events,
                   version=data_version)

    @staticmethod
    def l0_packet_to_event(packet, test_uuid,
                            collection='IDB',
                            device='S_C'):
        """
        Convert a RPW L0 packet dictionary into a Test event.

        :param packet: RPW L0 packet dictionary
                       (as returned by roc.film.tasks.l0.L0.l0_to_packet_list method)
        :param test_uuid: UUID of the test
        :param collection: Test event collection
        :param device: MEB GSE event device
        :return: Corresponding Test event
        """

        if packet['palisade_id'].startswith('TC'):
            packet_type = 'TC_FEEDBACK'
        elif packet['palisade_id'].startswith('TM'):
            packet_type = 'TM'
        else:
            logger.error(f'Unknown input packet type ({packet})!')
            packet_type = 'UNKNOWN'

        try:
            if packet['tc_exe_state'] == 'NOT_APPLICABLE':
                status = 'Unknown'
            elif packet['tc_exe_state'] == 'FAILED':
                status = 'Failed'
            elif packet['tc_exe_state'] == 'IDLE':
                status = 'Pending'
            elif packet['tc_exe_state'] == 'PASSED':
                status = 'Terminated'
            else:
                status = 'Unknown'
        except:
            status = None

        event_date = packet['utc_time'].strftime(TESTLOG_EVENTDATE_STRFORMAT)
        event_time = packet['utc_time'].strftime(TESTLOG_EVENTTIME_STRFORMAT)[:-3]

        # Monkey patch due to bug in L0 (should be solved, but let as is in case of)
        try:
            category = packet['category']
            if packet['category'][0] != '/' :
                category = '/'+packet['category']
        except:
            category = None

        eventdescr = {'EventDate': event_date,
                      'EventTime': event_time,
                      'Name': packet['palisade_id'],
                      'Data': packet['binary'],
                      'Device': device,
                      'TestUUID': test_uuid,
                      'EventType': packet_type,
                      'EventID': None, # Not defined at this stage
                      'Category': category,
                      'Collection': collection,
                      'Status': status
                      }

        return eventdescr

def parse_ads_file(input_file,
                   remove_header=None):
    """
    Parse ADS GSE text file.

    :param input_file: input ADS text file containing the TM/TC packets
    :param remove_header: header in bytes to remove
    :returns: list of packets dictionary with 'binary' and 'utc_time' keywords
    """

    # Initialize outputs
    packets = []

    if not os.path.isfile(input_file):
        raise FileNotFoundError(f'{input_file} NOT FOUND, ABORTING!')

    # TODO - Add timemin, timemax

    # Open input file
    with open(input_file, 'r') as file_buffer:
        # And extract data
        for i, line in enumerate(file_buffer):
            items = line.split(' ')
            if len(items) != 2:
                logger.warning(f'Line #{i} is not well formatted, skip it! ({line})')
                continue

            # Get time
            time = items[0].strip()
            if time == INVALID_ADS_TIME:
                logger.warning(f'Invalid time at line #{i}, skip it!')
                continue
            else:
                utc_time = dt.datetime.strptime(time, INPUT_TIME_STRFORMAT)

            # Get tm packet in hexadecimal
            binary = items[1].strip()
            if remove_header:
                binary = bytearray.fromhex(binary)[remove_header:].hex()

            # Add binary_packet and corresponding time to the outputs
            packets.append({'binary': binary, 'utc_time': utc_time})

    return packets


def is_excluded(category, included_categories=['/RPW', ]):
    """
    Says if a category with its subcategory can be safely considered as
    excluded from the list of path for categories to exclude.
    """
    path = Path(category)

    # convert to path
    included_path_categories = map(lambda x: Path(x), included_categories)

    return not any(
        map(
            lambda x: x in path.parents or x == path,
            included_path_categories,
        )
    )

def to_test_log_repr(test_log, test_file, descriptor):
    """
    Create a test log representation given the XML target and
    the XML descriptor element to place values inside the representation of the
    test log.

    :param test_log: Test object to insert
    :param xml: string containing test log file
    :param descriptor: test descriptor XML
    :return: TestLog representation
    """

    general_info = descriptor.find('GeneralInfo')
    execution_info = descriptor.find('ExecutionInformation')
    description = general_info.find('LongDescription')
    author = general_info.find('Author')

    return TestLog(
        file_parent=os.path.basename(test_file),
        test_name=descriptor.get('TestName'),
        test_id=descriptor.get('TestUUID'),
        test_creation_date=general_info.find('CreationDate').text,
        test_descr=description.text if description is not None else None,
        test_author=author.text if author is not None else None,
        test_status=execution_info.find('Status').text,
        test_launched=execution_info.find('LaunchedDate').text,
        test_terminated=execution_info.find('TerminatedDate').text,
        test_sha=test_log.sha1,
        test_insert_date=dt.datetime.today().strftime(TESTLOG_STRFTIME),
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""GUEST plugin tasks related to ROC database."""

import sys

from sqlalchemy import and_
from sqlalchemy.orm.exc import NoResultFound

from roc.guest.exceptions import ClearTestError, RocDbTransactionError

try:
    from poppy.core.logger import logger
    from poppy.core.db.connector import Connector
    from poppy.core.db.handlers import get_or_create_with_info
    from poppy.core.task import Task
    from poppy.core.target import FileTarget, PyObjectTarget
    from poppy.core.db.dry_runner import DryRunner
    from poppy.core.conf import settings
except:
    print('POPPy framework seems to not be installed properly!')

from roc.guest.models.test import TestLog
from roc.guest.models.packet import PacketLog
from roc.guest.guest import Test

__all__ = ['test_to_rocdb', 'clear_test']

# get the task test_to_db
@PyObjectTarget.input(identifier='raw_data')
@Task.as_task(plugin_name='roc.guest', name='test_to_rocdb')
@Connector.if_connected(settings.PIPELINE_DATABASE)
def test_to_rocdb(task):
    """
    To put into the ROC database a given Test object.
    """
    # get the test instance
    test_log = task.inputs['raw_data'].value

    # get the database session
    session = task.pipeline.db.session

    # Create file log entry in the ROC database from test log data
    file_log, created = get_or_create_with_info(
        session,
        TestLog,
        test_id=str(test_log.uuid),
        create_method_kwargs=test_log.to_testlog_repr(),
    )

    # If test log already saved in the database, skip the insertion
    if created == False:
        logger.info(f'Test {test_log.name} ({test_log.uuid}) already inserted into the database')
        return None

    try:
        file_log_id = file_log.id_file_log
    except:
        logger.error(f'Inserting test {test_log.name} ({test_log.uuid}) has failed!')
        return None

    # Extract test events as PacketLog entries
    if len(test_log.packet_list) > 0:
        # If packet_list exists, insert it
        packets = [Test.packet_to_packet_log(packet, file_log_id) for packet in test_log.packet_list]
    else:
        # Else just insert events as retrieved from test log
        logger.info(f'Inserting "as is" events from test {test_log.name} ({test_log.uuid})')
        packets = [Test.event_to_packet_log(event, file_log_id) for event in test_log.events]


    # commit to database if there are events to insert (those with data)
    if len(packets) > 0:
        engine = session.bind
        engine.execute(
            PacketLog.__table__.insert(),
            packets,
        )


@DryRunner.dry_run
@Task.as_task(plugin_name='roc.guest', name='clear_test')
@Connector.if_connected(settings.PIPELINE_DATABASE)
def clear_test(task):
    """
    Clear a test from the ROC database given a name and an UUID.
    """
    # get a session from the ROC database
    database = task.pipeline.db.get_database()

    # check the database is connected
    database.connectDatabase()

    # check that the connection is made
    if not database.connected:
        message = '{0} is not connected'.format(database)
        logger.error(message)
        raise RocDbTransactionError(message)

    # get the arguments
    filters = []

    if ('clear_all' in task.pipeline.args and
            task.pipeline.args.clear_all is True):
        filters.append(TestLog.test_name.like('%'))
    else:
        if task.pipeline.args.test_name is not None:
            filters.append(TestLog.test_name ==
                           task.pipeline.args.test_name[0])
        elif ('pattern' in task.pipeline.args and
                task.pipeline.args.pattern is not None):
            filters.append(TestLog.test_name.like(
                task.pipeline.args.pattern[0]))

        if task.pipeline.args.test_uuid is not None:
            filters.append(TestLog.test_id ==
                           task.pipeline.args.test_uuid[0])

        # If --end-time option is set, add an upper time limit
        if ('end_time' in task.pipeline.args and
                task.pipeline.args.end_time is not None):
            filters.append(TestLog.test_terminated <= task.pipeline.args.end_time[0])

        # If --start-time option is set, add an lower time limit
        if ('start_time' in task.pipeline.args and
                task.pipeline.args.start_time is not None):
            filters.append(TestLog.test_launched >= task.pipeline.args.start_time[0])

    # get the test(s) from the database
    with database.query_context() as session:
        try:
            tests = session.query(TestLog).filter(
                and_(*filters)).all()
        except NoResultFound:
            pass
        except:
            logger.exception(f'Removing test [{filters}] from ROC database has failed!')
            raise RocDbTransactionError

        if len(tests) == 0:
            logger.warning('No test found with input criteria in the ROC database!')
            return

        # now remove the test(s)
        for test in tests:
            logger.info('Removing test {0} from the ROC database...'.format(
                test[0].test_name))
            # from the database
            session.delete(test[0])
            session.commit()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""GUEST plugin tasks related to RPW L0 file handling."""
import uuid
from datetime import datetime

import sys
import os

from roc.guest.constants import DATA_VERSION, TIME_ISO_STRFORMAT
from roc.guest.tools import valid_data_version
from roc.guest.guest import Test

try:
    from poppy.core.logger import logger
    from poppy.core.task import Task
    from poppy.core.target import FileTarget, PyObjectTarget
except Exception:
    sys.exit('POPPy framework seems to not be installed properly!')

try:
    from roc.film.tools.metadata import init_l0_meta, get_spice_kernels
    from roc.film.tools.file_helpers import generate_filepath, get_output_dir
    from roc.film.tools.l0 import L0
except Exception:
    sys.exit('Dependencies are missing!')

__all__ = ['L0ToTest', 'TestToL0']


class L0ToTest(Task):
    """
    Parse input L0 and save content as a Test class instance.
    """

    plugin_name = 'roc.guest'
    name = 'l0_to_test'

    def add_targets(self):

        self.add_input(target_class=FileTarget,
                       identifier='l0_file',
                       filepath=self.get_l0_file())
        self.add_output(target_class=PyObjectTarget,
                        identifier='raw_data')

    def get_l0_file(self):
        try:
            return self.pipeline.get('l0_file', default=[None])[0]
        except Exception:
            pass

    def run(self):

        try:
            l0_file = self.inputs['l0_file'].filepath
        except Exception:
            logger.error('No input RPW L0 file found')
            return

        # Store input L0 data as a Test class instance into output target value
        self.outputs['raw_data'].value = Test.from_l0(l0_file)


class TestToL0(Task):
    """
    Save test class instance data into an output L0 file.
    """

    plugin_name = 'roc.guest'
    name = 'test_to_l0'

    def add_targets(self):

        self.add_input(target_class=PyObjectTarget,
                       identifier='raw_data')
        self.add_output(target_class=FileTarget,
                        identifier='l0_file')

    def setup_input(self):

        # Get/create list of well processed L0 files
        self.processed_files = self.pipeline.get(
            'processed_files', default=[], create=True)
        # Get/create list of failed DDS files
        self.failed_files = self.pipeline.get(
            'failed_files', default=[], create=True)

        # Get test data object
        self.test_data = self.inputs['raw_data'].value
        if not self.test_data:
            logger.warning('Stopping test_to_l0 task: No input raw_data provided')
            return False
        else:
            if self.test_data.version:
                self.data_version = self.test_data.version
            else:
                self.data_version = valid_data_version(
                    self.pipeline.get('data_version', default=[DATA_VERSION])[0])

        # If output directory not found, create it
        self.output_dir = get_output_dir(self.pipeline)
        if not os.path.isdir(self.output_dir):
            logger.debug(f'Making {self.output_dir}...')
            os.makedirs(self.output_dir)

        return True

    def run(self):

        # Initialize inputs
        if not self.setup_input():
            return

        # Initialize L0 metadata
        extra_attrs = {'Data_version': self.data_version,
                       'TIME_MIN': self.test_data.date.strftime(TIME_ISO_STRFORMAT),
                       'TIME_MAX': self.test_data.terminated_date.strftime(TIME_ISO_STRFORMAT),
                       'Generation_date': datetime.utcnow().isoformat(),
                       'File_ID': str(uuid.uuid4()),
                       }
        # Attempt to add SPICE kernels in L0 metadata
        try:
            sclk_file = get_spice_kernels(time_instance=self.test_data.packet_parser.time,
                                          pattern='solo_ANC_soc-sclk')
        except Exception:
            sclk_file = None

        if sclk_file:
            extra_attrs['SPICE_KERNELS'] = sclk_file[-1]
        else:
            logger.info('No SPICE SCLK kernel found!')

        # Initialize L0 metadata
        l0_metadata = init_l0_meta(self,
                                   extra_attrs=extra_attrs)

        # Generate L0 filepath
        l0_filepath = generate_filepath(self, l0_metadata, '.h5')

        # Write L0 file
        try:
            L0().to_hdf5(l0_filepath,
                         packet_parser=self.test_data.packet_parser,
                         metadata=l0_metadata)

        except Exception:
            logger.exception(f'Producing {l0_filepath} has failed!')
            self.failed_files.append(l0_filepath)
        else:
            self.processed_files.append(l0_filepath)
            logger.info(f'{l0_filepath} saved')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Reference documentation: ROC-GEN-SYS-NTT-00038-LES

"""
Database model for packet_log table.
"""

from poppy.core.db.non_null_column import NonNullColumn

from poppy.core.db.base import Base
from sqlalchemy import String, ForeignKey, UniqueConstraint, Index
from sqlalchemy.orm import relationship, validates, backref
from sqlalchemy.ext.associationproxy import association_proxy
from sqlalchemy.dialects.postgresql import (
    BIGINT,
    BOOLEAN,
    DOUBLE_PRECISION,
    ENUM,
    INTEGER,
    SMALLINT,
    TIMESTAMP,
)

__all__ = [
     'PacketLog',
]
class PacketLog(Base):
    """
    Class representation of the table for packet_log table in the ROC database.
    """

    id_packet_log = NonNullColumn(BIGINT(), primary_key=True)
    length = NonNullColumn(INTEGER(),
                            descr='Packet length in bytes')
    type = NonNullColumn(String(8),
                         descr='Packet type (TC or TM)')
    category = NonNullColumn(String(512),
                         descr='Packet PALISADE category')
    apid = NonNullColumn(INTEGER(), nullable=True,
                         descr='Packet APID')
    sync_flag = NonNullColumn(BOOLEAN, nullable=True,
                                    descr='TM packet time synchronization flag')
    utc_time = NonNullColumn(TIMESTAMP,
                             descr='Packet creation/execution UTC time')
    srdb_id = NonNullColumn(String(16),
                         descr='Packet name (SRDB ID)')
    palisade_id = NonNullColumn(String(256),
                          descr='Packet PALISADE ID')
    binary = NonNullColumn(String(),
                             descr='Packet raw binary data (in hexadecimal)')
    sha = NonNullColumn(String(), nullable=True,
                             descr='Packet sha (hexdigest)')
    idb_version = NonNullColumn(String(128), nullable=True,
                          descr='IDB version used to identify packet')
    idb_source = NonNullColumn(String(128), nullable=True,
                          descr='IDB source used to identify packet')
    creation_time = NonNullColumn(String(1024), nullable=True,
                                    descr='Packet creation time in CCSDS CUC format coarse:fine. For TM only.')
    ack_exe_state = NonNullColumn(String(16), nullable=True,
                                             descr='Packet acknowledgment'
                                                   'execution completion status. For TC only.')
    ack_acc_state = NonNullColumn(String(16), nullable=True,
                                   descr='TC packet acknowledgment acceptance status. For TC only.')
    sequence_name = NonNullColumn(String(16), nullable=True,
                         descr='Sequence name. For TC only.')
    unique_id = NonNullColumn(String(256), nullable=True,
                         descr='Unique ID. For TC only.')
    insertion_time = NonNullColumn(TIMESTAMP,
                                    descr='Packet insertion local time.')

    __tablename__ = 'packet_log'
    __table_args__ = (
        UniqueConstraint('utc_time', 'srdb_id'),
        {
            'schema': 'gse',
        }
    )

class InvalidPacketLog(Base):
    """
    Class representation of the table for invalid_packet_log table in the ROC database.
    """

    id_invalid_packet_log = NonNullColumn(BIGINT(), primary_key=True)
    idb_version = NonNullColumn(String(128), nullable=True,
                          descr='IDB version used to analyze packet')
    idb_source = NonNullColumn(String(128), nullable=True,
                          descr='IDB source used to analyze packet')
    apid = NonNullColumn(INTEGER(), nullable=True,
                         descr='Packet APID')
    binary = NonNullColumn(String(),
                             descr='Packet raw binary data (in hexadecimal)')
    comment = NonNullColumn(String(), nullable=True,
                          descr='Additional comment about why packet is invalid')
    insertion_time = NonNullColumn(TIMESTAMP,
                                    descr='Packet insertion local time.')
    utc_time = NonNullColumn(TIMESTAMP,
                             descr='Packet creation/execution UTC time', nullable=True)
    srdb_id = NonNullColumn(String(16),
                         descr='Packet name (SRDB ID)', nullable=True)
    sha = NonNullColumn(String(), nullable=True,
                             descr='Packet sha (hexdigest)')

    __tablename__ = 'invalid_packet_log'
    __table_args__ = (
        UniqueConstraint('binary'),
        {
            'schema': 'gse',
        }
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""GUEST plugin tasks to parse RPW TM/TC packet binary data."""

import sys
import uuid

try:
    from poppy.core.logger import logger
    from poppy.core.task import Task
    from poppy.core.target import PyObjectTarget
except Exception:
    sys.exit('POPPy framework seems to not be installed properly!')

# Import external Tasks, classes and methods (if any)
try:
    from roc.rpl.packet_parser import PacketParser
    from roc.rpl.time import Time
    from roc.rpl.constants import VALID_PACKET
except Exception:
    logger.exception('RPL plugin modules cannot be imported!')

__all__ = ['ParseTestPackets']


class ParseTestPackets(Task):
    """
    Parse RPW packet binaries in input Test class instance.
    Return Test class instance with unpacked RPW data
    """

    plugin_name = 'roc.guest'
    name = 'parse_test_packets'

    def add_targets(self):

        self.add_input(target_class=PyObjectTarget,
                       identifier='raw_data')
        self.add_output(target_class=PyObjectTarget,
                        identifier='raw_data')

    def setup_inputs(self):

        # Initialize Time instance
        self.time_instance = Time()

        # Pass input arguments for the Time instance
        self.time_instance.no_spice = self.pipeline.get(
            'no_spice', default=True)

        # Get IDB inputs
        self.idb_version = self.pipeline.get('idb_version', default=[None])[0]
        self.idb_source = self.pipeline.get('idb_source', default=[None])[0]

        # Get test data object
        self.test_data = self.inputs['raw_data'].value

        # Initialize PacketParser instance
        self.PacketParser = PacketParser

        # Get start_time/end_time
        self.start_time = self.pipeline.get('start_time', default=[None])[0]
        self.end_time = self.pipeline.get('end_time', default=[None])[0]

    def run(self):

        # Define task job ID (long and short)
        self.job_uuid = str(uuid.uuid4())
        self.job_id = self.job_uuid[:8]
        logger.info(f'Task {self.job_id} is starting')
        try:
            # Initialize task inputs
            self.setup_inputs()
        except Exception:
            logger.exception(f'Initializing inputs has failed for task {self.job_id}!')
            self.pipeline.exit()
            return

        # Parse RPW packets for the current test
        try:
            packet_count = len(self.test_data.packet_list)
            logger.info(f'Extracting {packet_count} RPW packets '
                        f'from {self.test_data.file_path}...    [{self.job_id}]')
            parser = self._parse_packet(self.test_data.packet_list)
        except Exception:
            logger.exception(f'Parsing RPW packets from {self.test_data.file_path} has failed!    [{self.job_id}]')
        else:
            # Get only valid packets
            valid_packets = self.PacketParser.packet_status(
                parser.parsed_packets,
                status=VALID_PACKET)
            n_valid = len(valid_packets)

            # Get only invalid packets
            invalid_packets = self.PacketParser.packet_status(
                parser.parsed_packets,
                status=VALID_PACKET, invert=True)
            n_invalid = len(invalid_packets)

            if n_invalid > 0:
                logger.error(f'{n_invalid} invalid TM/TC packets '
                             f'found in {self.test_data.file_path}!    [{self.job_id}]')

            # Check if valid packets are found
            if n_valid == 0:
                logger.info(f'No valid TM/TC packet '
                            f'found in {self.test_data.file_path}    [{self.job_id}]')
            else:
                logger.info(f'{n_valid} valid TM/TC packets '
                            f'found in {self.test_data.file_path}    [{self.job_id}]')

            # Fill raw_data packet list
            self.test_data.packet_parser = parser

        self.outputs['raw_data'].value = self.test_data

    def _parse_packet(self, packet_list):
        """
        Analyze input packets.

        :param packet_list:
        :return:
        """

        parser = None

        # Initialize packet_parser
        parser = self.PacketParser(
            idb_version=self.idb_version,
            idb_source=self.idb_source,
            time=self.time_instance,
        )

        # connect to add exception when packet analysis is bad
        parser.extract_error.connect(self.exception)

        # Analyse input RPW TM/TC packets
        parser.parse_packets(packet_list,
                             start_time=self.start_time,
                             end_time=self.end_time,
                             valid_only=False)

        return parser



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pkgutil

__all__ = []

# allow automatic loading of modules recursively (automatic registration of
# plugins and viewers)
__path__ = pkgutil.extend_path(__path__, __name__)
for importer, modname, ispkg in pkgutil.walk_packages(
    path=__path__,
    prefix=__name__ + '.'
):
    __import__(modname)

# vim: set tw=79 :



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Reference documentation
# ROC-GEN-SYS-NTT-00038-LES_Iss01_Rev02(Mission_Database_Description_Document)

"""
Database model for rpw file processing history tables.
"""

from sqlalchemy import String
from sqlalchemy.dialects.postgresql import BIGINT, \
    TIMESTAMP, ENUM

from poppy.core.db.base import Base
from poppy.core.db.non_null_column import NonNullColumn

__all__ = [
     'TestLog',
]


test_state_list = ['OK', 'WARNING', 'ERROR']
test_status_list = ['Running', 'Pending', 'Terminated']

test_state_enum = ENUM(*test_state_list, name='test_state_type', schema='gse')
test_status_enum = ENUM(*test_status_list, name='test_status_type', schema='gse')

class TestLog(Base):
    """
    Class representation of the table for test_log table in the ROC database.
    """


    id_test_log = NonNullColumn(BIGINT(),
                                primary_key=True)
    test_id = NonNullColumn(String(48),
                              descr='uuid of the test',
                              unique=True)
    test_name = NonNullColumn(String(512),
                              descr='Name the test')
    test_sha = NonNullColumn(String(512), nullable=True,
                              descr='SHA of the test')
    test_version = NonNullColumn(String(16), nullable=True,
                                 descr='Data version of the test (if any)')
    test_state = NonNullColumn(test_state_enum, nullable=True,
                                descr='State of the test. Possible values'
                                      f' are: {test_state_list}')
    test_status = NonNullColumn(test_status_enum,
                                descr='Status of the test. Possible values'
                                      f' are: {test_status_list}')
    test_creation_date = NonNullColumn(TIMESTAMP(), nullable=True,
                                       descr='Local date and time of the test '
                                             'creation',
                                       comment='')
    test_insert_date = NonNullColumn(TIMESTAMP(),
                                     descr='Local date and time of the test '
                                           'insertion in the database',
                                     nullable=True)
    test_descr = NonNullColumn(String(512), nullable=True,
                              descr='Description of the test')
    test_author = NonNullColumn(String(512), nullable=True,
                              descr='Author of the test')
    test_launched = NonNullColumn(TIMESTAMP(),
                                   descr='Start time of test')
    test_terminated = NonNullColumn(TIMESTAMP(),
                                 descr='End time of test')
    file_parent = NonNullColumn(String(512), nullable=True,
                                 descr='Parent file (if any)')

    __tablename__ = 'test_log'
    __table_args__ = (
        {
            'schema': 'gse',
        }
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import xml.etree.cElementTree as ET
import os.path as osp
import datetime
from poppy.core.logger import logger
import os


from poppy.core.tools.exceptions import print_exception
from poppy.core.db.handlers import get_or_create
from poppy.core.command import Command
from poppy.core.db.database import Database
from roc.guest.models.meb_gse.database import DatabaseInformation
from roc.guest.models.meb_gse_data.test import Descriptor
from roc.guest.models.meb_gse_data.event import Event
from roc.guest.models.meb_gse_data.device import Device
from roc.guest.models.meb_gse_data.parser import ParserCollection
from roc.guest.models.meb_gse_data.packet import PacketType
from roc.guest.meb_connector import MEBSelector
from poppy.pop.tools import paths




def generate_database_information(session, databases):
    """
    Generate the database informations.
    """
    logger.info("Generate the database informations")

    databases = [
        get_or_create(
            session,
            DatabaseInformation,
            DatabaseName=database,
        )
        for database in databases
    ]
    for database in databases:
        database.CreationDate = datetime.datetime.now()
        database.TerminatedDate = datetime.datetime.now()

    database.Current = True


def create_meb_databases(meb, name, number):
    # create the engine of the database
    meb.create_engine()

    # try closing the session to kill all current transactions
    try:
        meb.scoped_session.close_all()
    except Exception as e:
        logger.error("Can't close sessions on sqlalchemy")
        logger.error(e)
        raise e

    # connection
    conn = meb.engine.connect()

    # names for meb data databases
    databases = [
        "{0}_data_{1}".format(name, x) for x in range(1, number + 1)
    ]

    # drop and create all databases
    try:
        logger.info("Creating databases")
        conn.execute("COMMIT")
        conn.execute("DROP DATABASE IF EXISTS {0}".format(name))
        conn.execute("CREATE DATABASE {0}".format(name))

        # read the script
        with open(paths.from_scripts("create_meb.sql"), "r") as f:
            script = f.read()

        # execute the script for database
        conn.execute("USE {0}".format(name))
        conn.execute(script)

        # read the script
        with open(paths.from_scripts("create_meb_data.sql"), "r") as f:
            script = f.read()

        # loop over databases
        for database in databases:
            conn.execute(
                "DROP DATABASE IF EXISTS {0}".format(database)
            )
            conn.execute("CREATE DATABASE {0}".format(database))
            conn.execute("USE {0}".format(database))

            # execute the script
            conn.execute(script)

        #  conn.execute("USE {0}".format(current))
        conn.execute("COMMIT")
    except Exception as e:
        logger.error(e)
        raise e

    # return create databases names
    return databases


def data_to_meb(session, path):
    """
    Transfer the data of test logs presents in the path directory into the fake
    MEB database.
    """
    # loop over files presents recursively in the given path
    for root, dirs, files in os.walk(path):
        for file_ in files:
            if file_.endswith(".xml"):
                xml_to_database(session, osp.join(root, file_))


def xml_to_database(session, xml):
    """
    Open the given XML file and dump its structure into the MEB database.
    """
    # open wml file
    logger.info("Treating file {0}".format(xml))
    with open(xml, "r") as f:
        # list of events
        events = []

        # loop without parsing on the document
        for _, elem in ET.iterparse(f, ["end"]):
            # if this is a TestDescriptor field
            if elem.tag == "TestDescriptor":
                # create a descriptor
                test = create_descriptor(elem)
                session.add(test)
                session.commit()

                # clear the element to not keep the whole document in memory,
                # this really IMPORTANT to not have a lot of memory used for
                # large files to parse
                elem.clear()

            # if this an EventDescr
            elif elem.tag == "EventDescr":
                # create an event
                event = create_event(session, elem, test)
                events.append(event)

                # clear the element
                elem.clear()

        session.add_all(events)


def create_descriptor(node):
    """
    Create a descriptor and add it into memory.
    """
    # execution information
    exec_info = node.find("ExecutionInformation")

    # informations
    info = node.find("GeneralInfo")

    # node for the description
    description = info.find("LongDescription")

    # creation date
    date = info.find("CreationDate")

    test = Descriptor(
        TestName=node.get("TestName"),
        UUID=node.get("TestUUID"),
        Status=exec_info.find("Status").text,
        Author=info.find("Author").text if info.find("Author") else None,
        Description=description.text,
        CreationDate=date.text,
        OneShot=False if node.find("OneShotTestInformation") is None else True,
        Launched=exec_info.find("LaunchedDate").text,
        TerminatedDate=exec_info.find("TerminatedDate").text,
        XML=ET.tostring(node, encoding="utf-8"),
    )
    return test


def create_event(session, node, test):
    """
    Create an event in the database according to the content of the node in the
    XML test log file.
    """
    # the content node
    content = node.find("Content")

    # the common node
    common = node.find("Common")

    # get the device
    device = get_or_create(
        session,
        Device,
        DeviceName=node.get("Device"),
    )

    # get packet type
    packet_type = get_or_create(
        session,
        PacketType,
        Type=node.get("EventType"),
        IsInput=False,  # don't know what to put
    )

    # the collection
    collection = get_or_create(
        session,
        ParserCollection,
        CollectionName=content.find("Collection").text,
    )

    # create the event
    event = Event(
        DateProduction="T".join(
            [
                common.find("EventDate").text,
                common.find("EventTime").text
            ],
        ),
        XML=ET.tostring(node, encoding="utf-8"),
        device=device,
        packet_type=packet_type,
        test=test,
        Category=content.find("Category").text,
        Name=content.find("Name").text,
        collection=collection,
    )
    return event


class FakeDatabaseROC(Command):
    """
    Command to generate a fake database with tests and corresponding data.
    """
    __command__ = "fake_database"
    __command_name__ = "fake_database"
    __parent__ = "guest"
    __parent_arguments__ = ["base"]
    __help__ = "To generate a fake database for the MEB and the ROC"


class PopulateMEB(Command):
    """
    Command to populate the fake database.
    """
    __command__ = "fake_populate"
    __command_name__ = "populate"
    __parent__ = "fake_database"
    __parent_arguments__ = ["base"]
    __help__ = "Populate fake tables in specified databases"

    def add_arguments(self, parser):
        # argument to read the path to the configuration file to load
        parser.add_argument(
            "-m",
            '--meb',
            help="Name of the MEB database",
            type=str,
        )

        # prefix for the name of databases created
        parser.add_argument(
            '--name',
            help="Name of the database that will be created",
            type=str,
            default="roc_meb_test_db",
        )

        # number of databases to create
        parser.add_argument(
            '--number',
            help="Number of databases to create for tests",
            type=int,
            default=3,
        )

    def __call__(self, args):
        # get the database from the arguments
        meb = Database.manager[args.meb]

        # hack the database name
        meb.parameters["database"] = args.name

        # check connected
        meb.connectDatabase()

        # check it is connected
        if not meb.connected:
            logger.error("{0} not connected".format(meb))
            return

        # names for meb data databases to populate and store their informations
        databases = [
            "{0}_data_{1}".format(args.name, x)
            for x in range(1, args.number + 1)
        ]

        # generate databases
        session = meb.scoped_session()
        try:
            generate_database_information(session, databases)
            session.commit()

            # create meb selector
            selector = MEBSelector(meb)
            current = selector.current()
            current.connectDatabase()
            meb_data_session = current.scoped_session

            # put the xml data into the database
            data_to_meb(meb_data_session, paths.from_data())

        except:
            print_exception()


class CreateMEB(Command):
    """
    Command to create the databases of the MEB.
    """
    __command__ = "fake_meb_create"
    __command_name__ = "create_meb"
    __parent__ = "fake_database"
    __parent_arguments__ = []
    __help__ = "Create the databases for the MEB database"

    def add_arguments(self, parser):
        # argument to read the path to the configuration file to load
        parser.add_argument(
            "-m",
            '--meb',
            help="Name of the MEB database",
            type=str,
        )

        # prefix for the name of databases created
        parser.add_argument(
            '--name',
            help="Name of the database that will be created",
            type=str,
            default="roc_meb_test_db",
        )

        # number of databases to create
        parser.add_argument(
            '--number',
            help="Number of databases to create for tests",
            type=int,
            default=3,
        )

    def __call__(self, args):
        # get the database from the arguments
        meb = Database.manager[args.meb]

        # force no database
        meb.parameters["database"] = ""

        # check database available
        if not meb.is_available_with_error():
            logger.error("{0} not connected".format(meb))
            return

        # create the databases of the meb
        create_meb_databases(meb, args.name, args.number)


# vim: set tw=79 :



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Contains some useful methods for GUEST plugin.
"""


import argparse
from datetime import datetime
import logging
import os

from roc.guest.constants import INPUT_DATETIME_STRFTIME
from roc.guest.exceptions import GuestException

__all__ = ['valid_time', 'raise_error', 'valid_single_file', 'valid_data_version']

logger = logging.getLogger(__name__)

def raise_error(message, exception=GuestException):
    """Add an error entry to the logger and raise an exception."""
    logger.error(message)
    raise exception(message)

def valid_data_version(data_version):
    """
    Make sure to have a valid data version.

    :param data_version: integer or string containing the data version
    :return: string containing valid data version (i.e., 2 digits string)
    """
    try:
        if isinstance(data_version, list):
            data_version = data_version[0]
        data_version = int(data_version)
        return f"{data_version:02d}"
    except ValueError:
        raise_error(f"Input value for --data-version is not valid! ({data_version})")

def valid_single_file(file):
    """
    Make sure to have a valid single file.

    :param file: 1-element list or string containing the path to the file
    :return:
    """
    try:
        if isinstance(file, list):
            file = file[0]
        if os.path.isfile(file):
            return file
        else:
            raise FileNotFoundError
    except FileNotFoundError:
           raise_error(f"Input file not found! ({file})",
                       exception=FileNotFoundError)
    except ValueError:
        raise_error(f"Input file is not valid! ({file})",
                    exception=ValueError)
    except Exception as e:
        raise_error(f"Problem with input file! ({file})",
                    exception=e)

def valid_time(t, format=INPUT_DATETIME_STRFTIME):
    """
    Validate input datetime string format.

    :param t: input datetime string
    :param format: expected datetime string format
    :return: datetime object with input datetime info
    """
    if t:
        try:
            return datetime.strptime(t, format)
        except ValueError:
            raise_error(f"Not a valid datetime: '{t}'.",
                    exception=argparse.ArgumentTypeError)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""GUEST plugin tasks related to Packets handling."""

import sys
import uuid
from datetime import datetime

try:
    from poppy.core.logger import logger
    from poppy.core.db.connector import Connector
    from poppy.core.db.handlers import get_or_create_with_info
    from poppy.core.task import Task
    from poppy.core.target import FileTarget, PyObjectTarget
    from poppy.core.db.dry_runner import DryRunner
except:
    sys.exit("POPPy framework seems to not be installed properly!")

from edds_process.response.make import make_tmraw_xml, make_tcreport_xml

from roc.rpl.packet_parser import RawData

from roc.guest.exceptions import GuestNoInputError
from roc.guest.guest import Test
from roc.guest.constants import INVALID_UTC_DATETIME, TCREPORT_STRTFORMAT

__all__ = ['raw_to_test', 'raw_to_tmraw', 'raw_to_tcreport']

@DryRunner.dry_run
@PyObjectTarget.input(identifier='raw_data')
@PyObjectTarget.output(identifier='test')
@Task.as_task(plugin_name='roc.guest', name='raw_to_test')
def raw_to_test(task):
    """
    Generate a Test instance from an input RawData object.

    :param task:
    :return:
    """

    # Get RawData instance
    try:
        raw_data = task.inputs['raw_data'].value
    except:
        raise GuestNoInputError("No input RawData object defined, aborting!")

    # generate test uuid
    test_uuid = str(uuid.uuid4())

    # if test_name provided, then get it
    if "out_test_name" in task.pipeline.args:
        test_name = task.pipeline.args.out_test_name
    else:
        # Else use the test_uuid as default test_name
        test_name = test_uuid

    # if test_description provided, then get it
    if "out_test_descr" in task.pipeline.args:
        test_description = task.pipeline.args.out_test_descr
    else:
        # Else use a default test description
        test_description = f"Test generated by roc.guest on {datetime.today()}"

    # Get packet_utc_time from raw_data
    packet_utc_time = raw_data.packet_utc_time

    # Define launched_date
    if ('start_time' in task.pipeline.args and
            task.pipeline.args.start_time):
        launched_date = task.pipeline.args.start_time
    else:
        launched_date = min(packet_utc_time)

    # defined terminated_date
    if ('end_time' in task.pipeline.args and
            task.pipeline.args.end_time):
        terminated_date = task.pipeline.args.end_time
    else:
        terminated_date = max(packet_utc_time)

    # No test log file name defined...
    file_path = ""

    test_log = Test(
        test_name,
        launched_date,
        test_uuid=test_uuid,
        creation_date=launched_date, # we assume that creation_date = launched_date
        description=test_description,
        terminated_date=terminated_date,
        status='Terminated',
        file_path=file_path,
    )
    test_log.invalid_packet_list = raw_data.invalid_packet_list
    test_log.rpl = raw_data.rpl

    if raw_data.rpl.idb_source == "PALISADE":
        palisade_version = raw_data.rpl.idb_version
    else:
        palisade_version = None

    test_log.events = Test.packets_to_events(raw_data.packet_list,
                               palisade_version=palisade_version)

    # And insert it into the pipeline
    task.outputs['test'] = test_log

@PyObjectTarget.input(identifier='raw_data')
@FileTarget.output(identifier='dds_tmraw_xml')
@Task.as_task(plugin_name='roc.guest', name='raw_to_tmraw')
def raw_to_tmraw(task):
    """
    Save input RPW TM binary packet into a MOC DDS response TmRaw XML file

    :param task:
    :return:
    """
    # get the raw_data object from pipeline properties
    try:
        raw_data = task.inputs['raw_data'].value
    except:
        logger.warning("No input RawData object available, aborting!")
        return False

    # If failed_files list not defined in the pipeline properties, initialize it
    failed_files = task.pipeline.get('failed_files', default=[], create=True)

    # If processed_files list not defined in the pipeline properties, initialize it
    processed_files = task.pipeline.get('processed_files', default=[], create=True)

    # Get overwrite argument or define it
    overwrite = task.pipeline.get('overwrite', default=False, args=True)

    # Get data version (if any)
    data_version = task.pipeline.get('data_version', default=[None])[0]

    # Get output filepath or define it
    output_filepath = task.pipeline.get('output_tmraw_xml', default=None, args=True)
    if not output_filepath:
        if isinstance(raw_data, Test):
            output_filepath = raw_data.file_name_format(
                task.pipeline,
                "solo_LZ_rpw-dds-tmraw",
                overwrite=overwrite,
                data_version=data_version) + '.xml'
        else:
            output_filepath = "solo_LZ_rpw-dds-tmraw_" + str(uuid.uuid4())[:7] + '_V01.xml'

    # Add dummy scos header (as hexa string)
    if ('scos_header_size' in task.pipeline.args and
        task.pipeline.args.scos_header_size):
        scos_header = bytearray(task.pipeline.args.scos_header_size).hex()
    else:
        scos_header = None

    # Get TM packet only
    tm_packets = RawData.filter(raw_data.packet_list, by_type='TM')

    # Get binary
    binary_list = []
    for current_packet in tm_packets:
        current_binary = current_packet['binary']
        # If requested, add the scos header hexa before the packet binary
        if scos_header:
            current_binary = scos_header + current_binary

        binary_list.append(current_binary)

    logger.info(f'Writing {output_filepath}...')
    if make_tmraw_xml(binary_list, output_filepath,
                    overwrite=overwrite):
        logger.info("{0} saved".format(output_filepath))
        processed_files.append(output_filepath)
    else:
        logger.error(f'Saving {output_filepath} has failed')
        failed_files.append(output_filepath)

    task.outputs['dds_tmraw_xml'].filepath = output_filepath


@PyObjectTarget.input(identifier='raw_data')
@FileTarget.output(identifier='dds_tcreport_xml')
@Task.as_task(plugin_name='roc.guest', name='raw_to_tcreport')
def raw_to_tcreport(task):
    """
    Save input binary packet into a MOC DDS response TmRaw XML file

    :param task:
    :return:
    """
    # get the raw_data object from pipeline properties
    try:
       raw_data = task.inputs['raw_data'].value
    except:
       logger.warning("No input RawData object available, aborting!")
       return None

    # Get data version
    data_version = task.pipeline.get('data_version',
                                     default=[None])[0]

    # Get overwrite argument or define it
    overwrite = task.pipeline.get('overwrite', default=False, args=True)

    # If failed_files list not defined in the pipeline properties, initialize it
    failed_files = task.pipeline.get('failed_files', default=[], create=True)

    # If processed_files list not defined in the pipeline properties, initialize it
    processed_files = task.pipeline.get('processed_files', default=[], create=True)

    # Get output filepath or define it
    output_filepath = task.pipeline.get('output_tcreport_xml', default=None, args=True)
    if not output_filepath:
        if isinstance(raw_data, Test):
            output_filepath = raw_data.file_name_format(
                task.pipeline,
                "solo_LZ_rpw-dds-tcreport",
                data_version=data_version,
                overwrite=overwrite) + '.xml'
        else:
            output_filepath = "solo_LZ_rpw-dds-tcreport_" + str(uuid.uuid4())[:7] + '_V01.xml'

    # Get TM packets only
    tc_packets = RawData.filter(raw_data.packet_list, by_type='TC').copy()

    # Prepare output value as expected in tcreport
    for pkt in tc_packets:
        if pkt['utc_time'] != INVALID_UTC_DATETIME:
            pkt['utc_time'] = pkt['utc_time'].strftime(TCREPORT_STRTFORMAT)

    # Write Tc report
    logger.info(f'Writing {output_filepath}...')
    if make_tcreport_xml(tc_packets,
                         output_filepath,
                         overwrite=overwrite):
        logger.info("{0} saved".format(output_filepath))
        processed_files.append(output_filepath)
    else:
        logger.error(f'Saving {output_filepath} has failed')
        failed_files.append(output_filepath)

    task.outputs['dds_tcreport_xml'].filepath = output_filepath



# -*- coding: utf-8 -*-
"""empty message

Revision ID: roc_guest_0001_initial
Revises: poppy_pop_0001_initial
Create Date: 2020-11-20 16:24:09.119637

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from poppy.pop.alembic.helpers import create_table, create_schema, execute, user, drop_table

# revision identifiers, used by Alembic.
revision = 'roc_guest_0001_initial'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():

    create_schema('gse')  # added by hand

    ## delete the types if they exist
    execute('DROP TYPE IF EXISTS gse.test_state_type')
    execute('DROP TYPE IF EXISTS gse.test_status_type')

    # ### commands auto generated by Alembic - please adjust! ###
    create_table('invalid_packet_log',
    sa.Column('id_invalid_packet_log', sa.BIGINT(), nullable=False),
    sa.Column('idb_version', sa.String(length=128), nullable=True),
    sa.Column('idb_source', sa.String(length=128), nullable=True),
    sa.Column('apid', sa.INTEGER(), nullable=True),
    sa.Column('binary', sa.String(), nullable=False),
    sa.Column('comment', sa.String(), nullable=True),
    sa.Column('insertion_time', postgresql.TIMESTAMP(), nullable=False),
    sa.Column('utc_time', postgresql.TIMESTAMP(), nullable=True),
    sa.Column('srdb_id', sa.String(length=16), nullable=True),
    sa.Column('sha', sa.String(), nullable=True),
    sa.PrimaryKeyConstraint('id_invalid_packet_log'),
    sa.UniqueConstraint('binary'),
    schema='gse'
    )
    create_table('packet_log',
    sa.Column('id_packet_log', sa.BIGINT(), nullable=False),
    sa.Column('length', sa.INTEGER(), nullable=False),
    sa.Column('type', sa.String(length=8), nullable=False),
    sa.Column('category', sa.String(length=512), nullable=False),
    sa.Column('apid', sa.INTEGER(), nullable=True),
    sa.Column('sync_flag', sa.BOOLEAN(), nullable=True),
    sa.Column('utc_time', postgresql.TIMESTAMP(), nullable=False),
    sa.Column('srdb_id', sa.String(length=16), nullable=False),
    sa.Column('palisade_id', sa.String(length=256), nullable=False),
    sa.Column('binary', sa.String(), nullable=False),
    sa.Column('sha', sa.String(), nullable=True),
    sa.Column('idb_version', sa.String(length=128), nullable=True),
    sa.Column('idb_source', sa.String(length=128), nullable=True),
    sa.Column('creation_time', sa.String(length=1024), nullable=True),
    sa.Column('ack_exe_state', sa.String(length=16), nullable=True),
    sa.Column('ack_acc_state', sa.String(length=16), nullable=True),
    sa.Column('sequence_name', sa.String(length=16), nullable=True),
    sa.Column('unique_id', sa.String(length=256), nullable=True),
    sa.Column('insertion_time', postgresql.TIMESTAMP(), nullable=False),
    sa.PrimaryKeyConstraint('id_packet_log'),
    sa.UniqueConstraint('utc_time', 'srdb_id'),
    schema='gse'
    )
    create_table('test_log',
    sa.Column('id_test_log', sa.BIGINT(), nullable=False),
    sa.Column('test_id', sa.String(length=48), nullable=False),
    sa.Column('test_name', sa.String(length=512), nullable=False),
    sa.Column('test_sha', sa.String(length=512), nullable=True),
    sa.Column('test_version', sa.String(length=16), nullable=True),
    sa.Column('test_state', postgresql.ENUM('OK', 'WARNING', 'ERROR', name='test_state_type', schema='gse'), nullable=True),
    sa.Column('test_status', postgresql.ENUM('Running', 'Pending', 'Terminated', name='test_status_type', schema='gse'), nullable=False),
    sa.Column('test_creation_date', postgresql.TIMESTAMP(), nullable=True),
    sa.Column('test_insert_date', postgresql.TIMESTAMP(), nullable=True),
    sa.Column('test_descr', sa.String(length=512), nullable=True),
    sa.Column('test_author', sa.String(length=512), nullable=True),
    sa.Column('test_launched', postgresql.TIMESTAMP(), nullable=False),
    sa.Column('test_terminated', postgresql.TIMESTAMP(), nullable=False),
    sa.Column('file_parent', sa.String(length=512), nullable=True),
    sa.PrimaryKeyConstraint('id_test_log'),
    sa.UniqueConstraint('test_id'),
    schema='gse'
    )
    # ### end Alembic commands ###

    # added by hand
    # fix table permissions
    grant_user_seq = (
        'GRANT USAGE, SELECT ON ALL SEQUENCES IN ' 'SCHEMA gse TO {0}'.format(user)
    )
    execute(grant_user_seq)

def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    drop_table('test_log', schema='gse')
    drop_table('packet_log', schema='gse')
    drop_table('invalid_packet_log', schema='gse')
    # ### end Alembic commands ###

    ## delete the types if they exist
    execute('DROP TYPE IF EXISTS gse.test_state_type')
    execute('DROP TYPE IF EXISTS gse.test_status_type')



# -*- coding: utf-8 -*-

from roc.guest.tasks.mebdb import *
from roc.guest.tasks.txt import *
from roc.guest.tasks.xml import *
from roc.guest.tasks.rocdb import *
from roc.guest.tasks.raw import *
from roc.guest.tasks.l0 import *



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .base import Base
from sqlalchemy.orm import relationship
from sqlalchemy.ext.associationproxy import association_proxy
from sqlalchemy import Column, Integer, ForeignKey, String
from sqlalchemy.ext.declarative import DeferredReflection


__all__ = ["Code", "Message"]


class Message(DeferredReflection, Base):
    __tablename__ = "errormessage"

    errorCode = Column(Integer, primary_key=True)
    errorMessage = Column(String)

class Code(DeferredReflection, Base):
    __tablename__ = "errorcode"

    eventID = Column(Integer, ForeignKey("eventdescr.ID"), primary_key=True)
    error = relationship("Message", uselist=False)
    event = relationship("Event", uselist=False)
    message = association_proxy("error", "ErrorMessage")





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['roc',
 'roc.guest',
 'roc.guest.models',
 'roc.guest.models.meb_gse',
 'roc.guest.models.meb_gse_data',
 'roc.guest.models.versions',
 'roc.guest.tasks',
 'roc.guest.tests']

package_data = \
{'': ['*'], 'roc.guest': ['scripts/*', 'templates/*']}

install_requires = \
['Jinja2>=3.0,<4.0',
 'mysqlclient>=1.4.6,<2.0.0',
 'poppy-core',
 'poppy-pop',
 'roc-film>=1.0,<2.0',
 'roc-rpl>=1.0,<2.0']

setup_kwargs = {
    'name': 'roc-guest',
    'version': '1.3.0',
    'description': 'Gse data ReqUESTer (GUEST): Plugin to handle data products from/to ground tests (MEB GSE, ADS GSE)',
    'long_description': 'GUEST PLUGIN README\n===================\n\nThe RPW Gse data ReqUESTer (GUEST) is a plugin used to handle data from GSE for RPW/Solar Orbiter (MEB GSE, ADS GSE).\n\nGUEST is designed to be run in an instance of the ROC Ground Test SGSE (RGTS).\n\nGUEST is developed with and run under the POPPY framework.\n\n## Quickstart\n\n### Installation with pip\n\nTo install the plugin using pip:\n\n```\npip install roc-guest\n```\n\n### Installation from the repository (restricted access)\n\nFirst, retrieve the `GUEST` repository from the ROC gitlab server:\n\n```\ngit clone https://gitlab.obspm.fr/ROC/Pipelines/Plugins/GUEST.git\n```\n\nThen, install the package (here using (poetry)[https://python-poetry.org/]):\n\n```\npoetry install"\n```\n\nNOTES:\n\n    - It is also possible to clone the repository using SSH\n    - To install poetry: `pip install poetry`\n\n## Usage\n\nThe roc-guest plugin is designed to be run in a POPPy-built pipeline.\nNevertheless, it is still possible to import some classes and methods in Python files.\n\n## CONTACT\n\n* roc dot support at sympa dot obspm dot fr\n\n## License\n\n\nThis project is licensed under CeCILL-C.\n\n## Acknowledgments\n\nSolar Orbiter / RPW Operation Centre (ROC) team\n',
    'author': 'Xavier Bonnin',
    'author_email': 'xavier.bonnin@obspm.fr',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://gitlab.obspm.fr/ROC/Pipelines/Plugins/GUEST',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4',
}


setup(**setup_kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""GUEST plugin exceptions."""

__all__ = ['GuestException',
           'ParseTestXmlError',
           'ClearTestError',
            'GuestNoInputError',
           'MebDbTransactionError',
           'RocDbTransactionError']


class GuestException(Exception):
    """
    Guest main exception
    """

class ParseTestXmlError(Exception):
    """
    Errors for parsing xml test for guest module
    """

class ClearTestError(Exception):
    """
    Errors for clearing test for guest module
    """

class GuestNoInputError(Exception):
    """
    Errors for No input for guest module
    """

class MebDbTransactionError(Exception):
    """
    Errors with MEB DB transaction
    """

class RocDbTransactionError(Exception):
    """
    Errors with ROC DB transaction
    """



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .base import Base
from sqlalchemy.ext.declarative import DeferredReflection
from sqlalchemy import Column, Integer, String, DateTime
from sqlalchemy import Enum

__all__ = ["Descriptor"]

class Descriptor(Base):
	__tablename__ = "testdescriptor"

	ID = Column(Integer, primary_key=True)
	UUID = Column(String)
	TestName = Column(String)
	Status = Column('status', Enum('Running', 'Pending', 'Terminated', name='myenum'))
	Author = Column(String)
	Description = Column(String)
	CreationDate = Column(DateTime)
	OneShot = Column(Integer)
	Launched = Column(DateTime)
	TerminatedDate = Column(DateTime)
	RunningDPUIsNominal = Column(Integer)
	XML = Column(String)
	MainScript =  Column(Integer)
	HWVersion = Column(String)
	SWVersion = Column(String)
	OtherConfiguration = Column(String)
	Masked = Column(Integer)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from sqlalchemy import Column, Integer, String
from .base import Base
from sqlalchemy.ext.declarative import DeferredReflection

__all__ = ["DatabaseInformation"]


class DatabaseInformation(Base):
	__tablename__ = "databaseinformation"
	ID = Column(Integer, primary_key=True)
	DatabaseName = Column(String)
	Name = Column(String)
	Current = Column(Integer)